<?php
/**
 * $Horde: ansel/gallery/sort.php,v 1.5 2007/03/17 05:05:29 chuck Exp $
 *
 * Copyright 2001-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author Chuck Hagenbuch <chuck@horde.org>
 */

@define('ANSEL_BASE', dirname(__FILE__) . '/..');
require_once ANSEL_BASE . '/lib/base.php';

/* If we aren't provided with a gallery, redirect to the gallery
 * list. */
$galleryId = Util::getFormData('gallery');
if (!isset($galleryId)) {
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

$gallery = &$ansel_shares->getGallery($galleryId);
if (is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
    $notification->push(_("You do not have permission to change this gallery."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

switch (Util::getPost('action')) {
case 'Sort':
    parse_str(Util::getPost('order'), $order);
    $order = $order['order'];
    foreach ($order as $pos => $id) {
        $gallery->setImageOrder($id, $pos);
    }

    $notification->push(_("Gallery sorted."), 'horde.success');
    header('Location: ' . Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $galleryId), true));
    exit;
}

Horde::addScriptFile('prototype.js', 'horde', true);
Horde::addScriptFile('effects.js', 'ansel', true);
Horde::addScriptFile('dragdrop.js', 'ansel', true);
$title = sprintf(_("%s :: Sort"), $gallery->get('name'));
require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';
?>
<h1 class="header"><?php echo htmlspecialchars($title) ?></h1>

<div class="instructions">
 <form action="sort.php" method="post">
  <?php echo Util::formInput() ?>
  <input type="hidden" name="gallery" value="<?php echo (int)$galleryId ?>" />
  <input type="hidden" name="action" value="Sort" />
  <input type="hidden" name="order" id="order" />
  <p>
   <?php echo _("Drag images to the desired sort position.") ?>
   <input type="submit" onclick="$('order').value = Sortable.serialize('sortContainer', { name: 'order' });" class="button" value="<?php echo _("Done") ?>" />
  </p>
 </form>
</div>

<div id="sortContainer">

<?php
$images = $gallery->getImages();
foreach ($images as $image) {
    echo '<div id="o_' . $image->id . '"><p class="dropShadow"><a title="' .
        htmlspecialchars($image->caption) . '" href="#">' .
        Horde::img(Ansel::getImageUrl($image->id, 'thumb'), '', '', '') .
        '</a></p></div>';
}

echo '<br class="clear" /></div>';
$notification->push('Sortable.create(\'sortContainer\', {tag: \'div\', overlap: \'horizontal\', constraint: false })', 'javascript');
require $registry->get('templates', 'horde') . '/common-footer.inc';
