<?php
/**
 * $Horde: ansel/image.php,v 1.111 2007/01/02 12:46:46 jan Exp $
 *
 * Copyright 2003-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author Chuck Hagenbuch <chuck@horde.org>
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/Form.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Text.php';
require_once 'Horde/Text/Filter.php';
require_once 'Horde/Variables.php';

class ImageForm extends Horde_Form {

    var $_useFormToken = false;

    function ImageForm(&$vars, $title)
    {
        global $gallery;

        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Save"));
        $this->addHidden('', 'actionID', 'text', false);
        $this->addHidden('', 'gallery', 'text', false);
        $this->addHidden('', 'image', 'text', false);
        $this->addHidden('', 'page', 'text', false);

        $this->addVariable(_("Make this the default image for this gallery?"), 'image_default', 'boolean', false);
        $this->addVariable(_("Caption"), 'image_desc', 'longtext', false, false, null, array('4', '40'));
    }

}

class WatermarkForm extends Horde_Form {

    var $_useFormToken = false;

    function WatermarkForm(&$vars, $title)
    {
        global $gallery, $prefs;

        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Save"));
        $this->addHidden('', 'actionID', 'text', false);
        $this->addHidden('', 'gallery', 'text', false);
        $this->addHidden('', 'image', 'text', false);
        $this->addHidden('', 'page', 'text', false);

        $w = &$this->addVariable(_("Custom Watermark"), 'watermark', 'text', false, false, null);
        $w->setDefault($prefs->getValue('watermark'));

        $fonts = array('tiny' => _("Tiny"), 'small' => _("Small"), 'medium' => _("Medium"), 'large' => _("Large"), 'giant' => _("Giant"));
        $f = &$this->addVariable(_("Watermark Font"), 'font', 'enum', false, false, null, array($fonts));
        $f->setDefault('small');

        $ha = array('left' => _("Left"), 'center' => _("Center"), 'right' => _("Right"));
        $wha = &$this->addVariable(_("Horizontal Alignment"), 'whalign', 'enum', false, false, null, array($ha));
        $wha->setDefault('right');

        $va = array('top' => _("Top"), 'middle' => _("Middle"), 'bottom' => _("Bottom"));
        $wva = &$this->addVariable(_("Vertical Alignment"), 'wvalign', 'enum', false, false, null, array($va));
        $wva->setDefault('bottom');
    }

}

$actionID = Util::getFormData('actionID');
$gallery_id = Util::getFormData('gallery');
$image_id = Util::getFormData('image');
$page = Util::getFormData('page', 0);
$watermark_font = Util::getFormData('font');
$watermark_halign = Util::getFormData('whalign');
$watermark_valign = Util::getFormData('wvalign');

$watermark = Util::getFormData('watermark', $prefs->getValue('watermark'));
if ($watermark) {
    require_once 'Horde/Identity.php';
    $identity = &Identity::singleton();
    $name = $identity->getValue('fullname');
    if (empty($name)) {
        $name = Auth::getAuth();
    }

    // Set up array of possible substitutions.
    $watermark_array = array('%N' => $name,            // User's fullname.
                             '%L' => Auth::getAuth()); // User login.
    $watermark = str_replace(array_keys($watermark_array), array_values($watermark_array), $watermark);
    $watermark = strftime($watermark);
}

/* Redirect to the image list if no action has been requested. */
if (is_null($actionID)) {
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

$gallery = &$ansel_shares->getGallery($gallery_id);
if (is_a($gallery, 'PEAR_Error')) {
    $notification->push(_("Not found or error accessing gallery."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

/* Run through the action handlers. */
switch ($actionID) {
case 'modify':
    $image = &$ansel_shares->getImage($image_id);
    if ($image) {
        $title = sprintf(_("Edit properties :: %s"), $image->filename);

        /* Set up the form object. */
        $vars = Variables::getDefaultVariables();
        $vars->set('actionID', 'saveclose');
        $form = new ImageForm($vars, $title);
        $renderer = new Horde_Form_Renderer();

        /* Set up the gallery attributes. */
        $vars->set('image_default', $image->id == $gallery->get('default'));
        $vars->set('image_desc', $image->caption);
    }

    require ANSEL_TEMPLATES . '/common-header.inc';
    $form->renderActive($renderer, $vars, 'image.php', 'post', 'multipart/form-data');
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;

case 'saveclose':
case 'save':
    $title = _("Save Image");

    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(sprintf(_("Access denied saving image to %s."), $gallery->get('name')), 'horde.error');
        $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'view' => 'Gallery', 'page' => $page));
        header('Location: ' . Horde::applicationUrl($imageurl, true));
        exit;
    }

    /* Validate the form object. */
    $vars = Variables::getDefaultVariables();
    $vars->set('actionID', 'save');

    $renderer = new Horde_Form_Renderer();
    $form = new ImageForm($vars, _("Edit an image"));
    /* Update existing image. */
    if ($form->validate($vars)) {
        $image = &$ansel_shares->getImage($image_id);
        $image->caption = $vars->get('image_desc');
        $image->save();

        if ($vars->get('image_default')) {
            $gallery->set('default', $image_id);
            $gallery->set('default-type', 'manual', true);
        }

        if ($actionID == 'save') {
            /* Return to the image view. */
            $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'image' => $image_id, 'view' => 'Image', 'page' => $page));
            header('Location: ' . Horde::applicationUrl($imageurl, true));
        } else {
            Util::closeWindowJS(' window.opener.location.href = window.opener.location.href;');
        }
        exit;
    }
    break;

case 'editimage':
    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(_("You are not allowed to edit this image."), 'horde.error');

        /* Return to the image view. */
        $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'image' => $image_id, 'view' => 'Image', 'page' => $page));
        header('Location: ' . Horde::applicationUrl($imageurl, true));
        exit;
    }

    /* Retrieve image details. */
    $image = &$ansel_shares->getImage($image_id);
    $title = sprintf(_("Edit %s :: %s"), $gallery->get('name'), $image->filename);

    require ANSEL_TEMPLATES . '/common-header.inc';
    require ANSEL_TEMPLATES . '/menu.inc';
    require ANSEL_TEMPLATES . '/image/edit_image.inc';
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;

case 'rotate90':
case 'rotate180':
case 'rotate270':
case 'flip':
case 'mirror':
case 'grayscale':
case 'watermark':
    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(sprintf(_("Access denied saving image to %s."), $gallery->get('name')), 'horde.error');
    } else {
        $view = 'full';
        $image = &$ansel_shares->getImage($image_id);

        switch ($actionID) {
        case 'rotate90':
        case 'rotate180':
        case 'rotate270':
            $angle = intval(substr($actionID, 6));
            $image->rotate($view, $angle);
            break;

        case 'flip':
            $image->flip($view);
            break;

        case 'mirror':
            $image->mirror($view);
            break;

        case 'grayscale':
            $image->grayscale($view);
            break;

        case 'watermark':
            $image->watermark($view, $watermark, $watermark_halign, $watermark_valign, $watermark_font);
            break;
        }

        $image->updateData($image->raw());
    }

    $imageurl = Util::addParameter('image.php', array('gallery' => $gallery_id, 'image' => $image_id, 'actionID' => 'editimage', 'page' => $page));
    header('Location: ' . Horde::applicationUrl($imageurl, true));
    exit;

case 'setwatermark':
    $title = _("Watermark");
    $image = &$ansel_shares->getImage($image_id);
    if ($image) {
        /* Set up the form object. */
        $vars = Variables::getDefaultVariables();
        $vars->set('actionID', 'previewcustomwatermark');
        $form = new WatermarkForm($vars, _("Watermark"));
        $renderer = new Horde_Form_Renderer();
    }
    require ANSEL_TEMPLATES . '/common-header.inc';
    $form->renderActive($renderer, $vars, 'image.php', 'post');
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;

case 'previewcustomwatermark':
    $imageurl = Util::addParameter('image.php', array('gallery' => $gallery_id, 'image' => $image_id, 'page' => $page, 'watermark' => $watermark, 'font' => $watermark_font, 'whalign' => $watermark_halign, 'wvalign' => $watermark_valign, 'actionID' => 'previewwatermark'));
    $url = Horde::applicationUrl($imageurl);
    $url = str_replace('&amp;', '&', $url);
    Util::closeWindowJS('window.opener.location.href = "' . $url . '";');
    exit;

case 'previewgrayscale':
case 'previewwatermark':
case 'previewflip':
case 'previewmirror':
case 'previewrotate90':
case 'previewrotate180':
case 'previewrotate270':
    $title = _("Edit Image");
    $action = substr($actionID, 7);

    /* If the requested gallery doesn't exist, display an error
     * message. */
    if (is_a($gallery, 'PEAR_Error')) {
        $notification->push(_("Gallery not found."), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    } else {
        /* Retrieve image details. */
        $image = &$ansel_shares->getImage($image_id);
        $title = sprintf(_("Preview changes for %s :: %s"), $gallery->get('name'), $image->filename);

        require ANSEL_TEMPLATES . '/common-header.inc';
        require ANSEL_TEMPLATES . '/menu.inc';
        require ANSEL_TEMPLATES . '/image/preview_image.inc';
        require $registry->get('templates', 'horde') . '/common-footer.inc';
        exit;
    }
    break;

case 'imagerotate90':
case 'imagerotate180':
case 'imagerotate270':
    $view = Util::getFormData('view');
    $angle = intval(substr($actionID, 11));
    $image = &$ansel_shares->getImage($image_id);
    $image->rotate($view, $angle);
    $image->display($view);
    exit;

case 'imageflip':
    $view = Util::getFormData('view');
    $image = &$ansel_shares->getImage($image_id);
    $image->flip($view);
    $image->display($view);
    exit;

case 'imagemirror':
    $view = Util::getFormData('view');
    $image = &$ansel_shares->getImage($image_id);
    $image->mirror($view);
    $image->display($view);
    exit;

case 'imagegrayscale':
    $view = Util::getFormData('view');
    $image = &$ansel_shares->getImage($image_id);
    $image->grayscale($view);
    $image->display($view);
    exit;

case 'imagewatermark':
    $view = Util::getFormData('view');
    $image = &$ansel_shares->getImage($image_id);
    $image->watermark($view, $watermark, $watermark_halign, $watermark_valign, $watermark_font);
    $image->display($view);
    exit;

case 'delete':
    if (is_array($image_id)) {
        $images = array_keys($image_id);
    } else {
        $images = array($image_id);
    }

    /* Delete the images if we're provided with a valid image ID. */
    if (count($images)) {
        foreach ($images as $image) {
            if (!$gallery->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
                $notification->push(sprintf(_("Access denied deleting image %s."), $image), 'horde.error');
            } else {
                $result = $gallery->removeImage($image, true);
                if (is_a($result, 'PEAR_Error')) {
                    $notification->push(sprintf(_("There was a problem deleting images: %s"), $result->getMessage()), 'horde.error');
                } else {
                    $notification->push(_("Deleted the image."), 'horde.success');
                }
            }
        }
    }

    /* Return to the image list. */
    $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'view' => 'Gallery', 'page' => $page));
    header('Location: ' . Horde::applicationUrl($imageurl, true));
    exit;

case 'move':
    if (is_array($image_id)) {
        $images = array_keys($image_id);
    } else {
        $images = array($image_id);
    }

    /* Move the images if we're provided with at least one valid image
     * ID. */
    $newGallery = Util::getFormData('new_gallery');
    if ($images && $newGallery) {
        $newGallery = &$ansel_shares->getGallery($newGallery);
        if (is_a($newGallery, 'PEAR_Error')) {
            $notification->push(_("Bad input."), 'horde.error');
        } else {
            $result = $gallery->moveImagesTo($images, $newGallery);
            if (is_a($result, 'PEAR_Error')) {
                $notification->push($result, 'horde.error');
            } else {
                if ($result == 1) {
                    $notification->push(sprintf(_("Moved one image to %s"), $newGallery->get('name')), 'horde.success');
                } else {
                    $notification->push(sprintf(_("Moved %d images to %s"), $result, $newGallery->get('name')), 'horde.success');
                }
            }
        }
    }

    /* Return to the image list. */
    $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'view' => 'Gallery', 'page' => $page));
    header('Location: ' . Horde::applicationUrl($imageurl, true));
    exit;

default:
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';
$form->renderActive($renderer, $vars, 'image.php', 'post', 'multipart/form-data');
require $registry->get('templates', 'horde') . '/common-footer.inc';
