<?php
/**
 * $Horde: ansel/img/upload.php,v 1.9 2007/01/02 12:46:47 jan Exp $
 *
 * Copyright 2003-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author Chuck Hagenbuch <chuck@horde.org>
 */

@define('ANSEL_BASE', dirname(__FILE__) . '/..');
require_once ANSEL_BASE . '/lib/base.php';
require_once ANSEL_BASE . '/lib/Forms/Upload.php';
require_once 'Horde/Form/Renderer.php';
require_once 'Horde/Variables.php';

$gallery_id = Util::getFormData('gallery');
$gallery = &$ansel_shares->getGallery($gallery_id);
if (is_a($gallery, 'PEAR_Error')) {
    $notification->push(_("Not found or error accessing gallery."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

$page = Util::getFormData('page', 0);
$vars = Variables::getDefaultVariables();

$form = new UploadForm($vars, _("Upload new images"));
if ($form->validate($vars)) {
    $valid = true;
    $uploaded = 0;
    $form->getInfo($vars, $info);
    for ($i = 0; $i <= $conf['image']['num_uploads'] + 1; ++$i) {
        if (empty($info['file' . $i]['file'])) {
            continue;
        }

        /* Save new image. */
        if (!is_a(Browser::wasFileUploaded('file' . $i), 'PEAR_Error') &&
            filesize($info['file' . $i]['file'])) {

            /* Read in the uploaded data. */
            $data = file_get_contents($info['file' . $i]['file']);

            /* Check for a compressed file. */
            require_once 'Horde/MIME/Magic.php';
            if (in_array($info['file' . $i]['type'],
                         array('x-extension/zip',
                               'application/x-compressed',
                               'application/x-zip-compressed',
                               'application/zip')) ||
                MIME_Magic::filenameToMIME($info['file' . $i]['name']) == 'application/zip') {

                /* Get the list of files in the zipfile. */
                require_once 'Horde/Compress.php';
                $zip = &Horde_Compress::singleton('zip');
                $files = $zip->decompress($data, array('action' => HORDE_COMPRESS_ZIP_LIST));

                if (is_a($files, 'PEAR_Error')) {
                    $notification->push(sprintf(_("There was an error processing the uploaded archive: %s"), $files->getMessage()), 'horde.error');
                    continue;
                }

                foreach ($files as $key => $zinfo) {
                    // Skip some known metadata files.
                    $len = strlen($zinfo['name']);
                    if ($zinfo['name'][$len - 1] == '/') {
                        continue;
                    }
                    if (strrpos($zinfo['name'], '.DS_Store') == ($len - 9)) {
                        continue;
                    }
                    if (strpos($zinfo['name'], '__MACOSX/') !== false) {
                        continue;
                    }

                    $zdata = $zip->decompress($data, array('action' => HORDE_COMPRESS_ZIP_DATA,
                                                           'info' => $files,
                                                           'key' => $key));
                    if (is_a($zdata, 'PEAR_Error')) {
                        $notification->push(sprintf(_("There was an error processing the uploaded archive: %s"), $zdata->getMessage()), 'horde.error');
                        break;
                    }

                    /* If we successfully got data, try adding the
                     * image to the gallery. */
                    $image = array('image_filename' => $zinfo['name'],
                                   'image_caption' => '',
                                   'data' => $zdata);
                    $image_id = $gallery->addImage($image);
                    if (!is_a($image_id, 'PEAR_Error')) {
                        ++$uploaded;
                    } else {
                        $notification->push(sprintf(_("There was a problem saving the image: %s"), $image_id), 'horde.error');
                    }
                }
            } else {
                /* Try and make sure the image is in a recognizeable
                 * format. */
                if (getimagesize($info['file' . $i]['file']) === false) {
                    $notification->push(_("The file you uploaded does not appear to be a valid image."), 'horde.error');
                    continue;
                }

                $image_id = $gallery->addImage(array('image_filename' => $info['file' . $i]['name'],
                                                     'image_caption' => $vars->get('image' . $i . '_desc'),
                                                     'image_type' => $info['file' . $i]['type'],
                                                     'data' => $data));
                if (is_a($image_id, 'PEAR_Error')) {
                    $notification->push(sprintf(_("There was a problem saving the image: %s"), $image_id->getMessage()), 'horde.error');
                    $valid = false;
                } else {
                    ++$uploaded;
                    if ($vars->get('image_default')) {
                        $gallery->set('default', $image_id);
                        $gallery->set('default-type', 'manual', true);
                    }
                }
            }
        } else {
            if (!empty($info['file' . $i]['error'])) {
                $notification->push(sprintf(_("There was a problem uploading the image: %s"), $info['file' . $i]['error']), 'horde.error');
            } elseif (!filesize($info['file' . $i]['file'])) {
                $notification->push(_("The uploaded file appears to be empty. It may not exist on your computer."), 'horde.error');
            }
            $valid = false;
        }
    }

    /* Tell the user how well we did. */
    if ($uploaded == 1) {
        $notification->push(_("1 image was uploaded."), 'horde.success');
    } elseif ($uploaded) {
        $notification->push(sprintf(_("%d images were uploaded."), $uploaded), 'horde.success');
    }

    if ($valid) {
        /* Return to the gallery view. */
        $imageurl = Util::addParameter('view.php', array('gallery' => $gallery_id, 'view' => 'Gallery', 'page' => $page));
        header('Location: ' . Horde::applicationUrl($imageurl, true));
        exit;
    }
}

$title = _("Add Image");
require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';
$renderer = new Horde_Form_Renderer();
$form->renderActive($renderer, $vars, 'upload.php', 'post', 'multipart/form-data');
if ($browser->getPlatform() == 'win') {
    echo '<div class="box" style="margin:8px; padding:8px"><h3>' . Help::link('ansel', 'xppublish') . ' ' . _("Learn how to publish images directly from Windows.") . '</h3></div>';
}
require $registry->get('templates', 'horde') . '/common-footer.inc';
