/**
 * slidecontroller still needs to be able to update:
- breadcrumbs (last element set to url to image, with proper gallery page param)
- image links (download full, edit, properties, delete)
 *
 * You can only have one SlideController on a page.
 *
 * $Horde: ansel/js/src/slideshow.js,v 1.1 2006/06/11 04:13:30 chuck Exp $
 */
var SlideController = {

    photos: null,
    photoId: 0,

    slide: null,

    interval: null,
    intervalSeconds: 5,

    /**
     * CSS border size x 2
     */
    borderSize: 0,

    /**
     * Initialization.
     */
    initialize: function(photos, start) {
        SlideController.photoId = start || 0;

        Event.observe(window, 'load', function() {
            Event.observe($('Photo'), 'load', function() {
                SlideController.photo.showPhoto();
            });

            SlideController.photos = photos;
            SlideController.photo = new Slide(SlideController.photoId);
            SlideController.photo.initSwap();
        });
    },

    /**
     */
    play: function() {
        Element.hide('ssPlay');
        Element.show('ssPause');

        if (this.interval) {
            clearInterval(this.interval);
        }
        this.interval = setInterval(SlideController.next, SlideController.intervalSeconds * 1000);
    },

    /**
     */
    pause: function() {
        Element.hide('ssPause');
        Element.show('ssPlay');

        if (this.interval) {
            clearInterval(this.interval);
        }
    },

    /**
     */
    prev: function() {
        SlideController.photo.prevPhoto();
    },

    /**
     */
    next: function() {
        SlideController.photo.nextPhoto();
    }

}

// -----------------------------------------------------------------------------------
//
// This page coded by Scott Upton
// http://www.uptonic.com | http://www.couloir.org
//
// This work is licensed under a Creative Commons License
// Attribution-ShareAlike 2.0
// http://creativecommons.org/licenses/by-sa/2.0/
//
// Associated APIs copyright their respective owners
//
// -----------------------------------------------------------------------------------
// --- version date: 11/28/05 --------------------------------------------------------

/**
 * Additional methods for Element added by SU, Couloir.
 */
Object.extend(Element, {
    getWidth: function(element) {
        element = $(element);
        return element.offsetWidth;
    },
    setWidth: function(element,w) {
        element = $(element);
        element.style.width = w + 'px';
    },
    setSrc: function(element,src) {
        element = $(element);
        element.src = src;
    },
    /*setHref: function(element,href) {
        element = $(element);
        element.href = href;
    },*/
    setInnerHTML: function(element,content) {
        element = $(element);
        element.innerHTML = content;
    }
});

var Slide = Class.create();
Slide.prototype = {
    initialize: function(photoId) {
        this.photoId = photoId;
        this.photo = 'Photo';
        this.photoBox = 'Container';
        //this.prevLink = 'PrevLink';
        //this.nextLink = 'NextLink';
        this.captionBox = 'CaptionContainer';
        this.caption = 'Caption';
    },
    getCurrentSize: function() {
        // Get current height and width, subtracting CSS border size.
        this.wCur = Element.getWidth(this.photoBox) - SlideController.borderSize;
        this.hCur = Element.getHeight(this.photoBox) - SlideController.borderSize;
    },
    getNewSize: function() {
        // Get current height and width.
        this.wNew = SlideController.photos[SlideController.photoId][1];
        this.hNew = SlideController.photos[SlideController.photoId][2];
    },
    getScaleFactor: function() {
        this.getCurrentSize();
        this.getNewSize();
        // Scalars based on change from old to new.
        this.xScale = (this.wNew / this.wCur) * 100;
        this.yScale = (this.hNew / this.hCur) * 100;
    },
    setNewPhotoParams: function() {
        // Set source of new image.
        Element.setSrc(this.photo, SlideController.photos[SlideController.photoId][0]);

        // Set anchors for bookmarking.
        //Element.setHref(this.prevLink, "#" + (SlideController.photoId + 1));
        //Element.setHref(this.nextLink, "#" + (SlideController.photoId + 1));

        // Add caption from gallery array.
        Element.setInnerHTML(this.caption, SlideController.photos[SlideController.photoId][4]);

        // Set titles.
        Element.setInnerHTML('PhotoName', SlideController.photos[SlideController.photoId][3]);
        try {
            document.title = document.title.replace(SlideController.photos[this.photoId][3],
                                                    SlideController.photos[SlideController.photoId][3]);
            if (parent.frames.horde_main) {
                parent.document.title = document.title;
            }
        } catch (e) {}
    },
    resizePhotoBox: function() {
        this.getScaleFactor();
        new Effect.Scale(this.photoBox, this.yScale, { scaleX: false, duration: 0.3, queue: 'front' });
        new Effect.Scale(this.photoBox, this.xScale, { scaleY: false, delay: 0.5, duration: 0.3 });
        // Dynamically resize caption box as well.
        Element.setWidth(this.captionBox, this.wNew - (-SlideController.borderSize));
    },
    showPhoto: function() {
        new Effect.Appear(this.photo, { duration: 0.5, queue: 'end', afterFinish: (function() { Element.show(this.captionBox); }).bind(this) });
    },
    nextPhoto: function() {
        // Figure out which photo is next.
        (SlideController.photoId == (SlideController.photos.length - 1)) ? SlideController.photoId = 0 : ++SlideController.photoId;
        this.initSwap();
    },
    prevPhoto: function() {
        // Figure out which photo is previous.
        (SlideController.photoId == 0) ? SlideController.photoId = SlideController.photos.length - 1 : --SlideController.photoId;
        this.initSwap();
    },
    initSwap: function() {
        // Begin by hiding main elements.
        Element.hide(this.photo);
        Element.hide(this.captionBox);

        // Set new dimensions and source, then resize.
        this.setNewPhotoParams();
        this.resizePhotoBox();

        // Update the current photo id.
        this.photoId = SlideController.photoId;
    }
}
