<?php

$block_name = _("Gallery");

/**
 * Ansel's Gallery
 *
 * $Horde: ansel/lib/Block/gallery.php,v 1.10 2006/12/02 07:13:54 chuck Exp $
 *
 * This file provides a selected Ansel gallery through the Horde_Blocks, by
 * extending the Horde_Blocks class.
 *
 * Modified from random_gallery.php by Marcus Ryan <marcus@horde.org>
 * Original copyright 2003-2005 Duck <duck@obla.net>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Duck <Duck@obla.net>
 * @package Horde_Block
 */
class Horde_Block_ansel_gallery extends Horde_Block {

    var $_app = 'ansel';
    var $_gallery = null;

    function _params()
    {
        require_once dirname(__FILE__) . '/../base.php';

        $params = array('gallery' => array(
                            'name' => _("Gallery"),
                            'type' => 'enum',
                            'default' => '__random',
                            'values' => array('__random' => _("Random Gallery"))));
        foreach (Ansel::listGalleries() as $id => $gal) {
            $params['gallery']['values'][$gal->getId()] = $gal->get('name');
        }

        return $params;
    }

    function _title()
    {
        $gallery = &$this->_getGallery();
        if (is_a($gallery, 'PEAR_Error')) {
            return Horde::link(Horde::applicationUrl('list.php', true)) .
                _("Gallery") . '</a>';
        }

        // Build the gallery name.
        if (isset($this->_params['gallery']) &&
            $this->_params['gallery'] == '__random') {
            $name = _("Random Gallery") . ': ' . $gallery->get('name');
        } else {
            $name = $gallery->get('name');
        }

        $viewurl = Util::addParameter(Horde::applicationUrl('view.php', true), 'gallery', $gallery->getId());

        return Horde::link($viewurl) . $name . '</a>';
    }

    function _content()
    {
        $gallery = &$this->_getGallery();
        if (is_a($gallery, 'PEAR_Error')) {
            return $gallery->getMessage();
        }

        $viewurl = Util::addParameter(Horde::applicationUrl('view.php', true), 'gallery', $gallery->getId());

        $html = '<div class="dropShadow">';
        $html .= Horde::link($viewurl, sprintf(_("View %s"), $gallery->get('name')));

        if ($gallery->getDefaultImage()) {
            $html .= Horde::img(Ansel::getImageUrl($gallery->getDefaultImage(), 'thumb', true), '', '', '');
        } else {
            $html .= Horde::img($GLOBALS['registry']->getImageDir() . '/thumb-error.png', '', '', '');
        }

        return $html . '</a></div>';
    }

    function &_getGallery()
    {
        @define('ANSEL_BASE', dirname(__FILE__) . '/../..');
        require ANSEL_BASE . '/lib/base.php';

        // Make sure we haven't already selected a gallery.
        if (is_a($this->_gallery, 'Ansel_Gallery')) {
            return $this->_gallery;
        }

        // Get the gallery object and cache it.
        if (isset($this->_params['gallery']) && $this->_params['gallery'] != '__random') {
            $this->_gallery = &$GLOBALS['ansel_shares']->getGallery($this->_params['gallery']);
        } else {
            $this->_gallery = &Ansel::getRandomGallery();
        }

        if (empty($this->_gallery)) {
            return PEAR::raiseError(_("Gallery does not exist."));
        } elseif (is_a($this->_gallery, 'PEAR_Error') ||
                  !$this->_gallery->hasPermission(Auth::getAuth(), PERMS_READ)) {
            return PEAR::raiseError(_("You do not have permission to view this gallery."));
        }

        // Return a reference to the gallery.
        return $this->_gallery;
    }

}
