<?php
/**
 * Exifer
 * Extracts EXIF information from digital photos.
 *
 * $Horde: ansel/lib/Exif/canon.php,v 1.7 2006/12/26 00:45:27 chuck Exp $
 *
 * Copyright 2003 Jake Olefsky
 * http://www.offsky.com/software/exif/index.php
 * jake@olefsky.com
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * http://www.gnu.org/copyleft/gpl.html
 */

/**
 * Looks up the name of the tag for the MakerNote (Depends on Manufacturer)
 *
 * @param string $tag  The tag code.
 *
 * @return string  The name for tag.
 */
function lookup_Canon_tag($tag)
{
    switch ($tag) {
    case '0001': return 'Settings 1';
    case '0004': return 'Settings 4';
    case '0006': return 'ImageType';
    case '0007': return 'FirmwareVersion';
    case '0008': return 'ImageNumber';
    case '0009': return 'OwnerName';
    case '000c': return 'CameraSerialNumber';
    case '000f': return 'CustomFunctions';
    default: return 'Unknown: ' . $tag;
    }
}

/**
 * Formats Data for the data type
 *
 * @param string $type
 * @param string $tag
 * @param integer|boolean $intel
 * @param string $data
 * @param array &$result
 */
function formatCanonData($type, $tag, $intel, $data, &$result)
{
    $place = 0;

    if ($type == 'ASCII') {
        $result = $data;
    } elseif ($type == 'URATIONAL' || $type == 'SRATIONAL') {
        $data = bin2hex($data);
        if ($intel == 1) {
            $data = intel2Moto($data);
        }
        $top = hexdec(substr($data, 8, 8));
        $bottom = hexdec(substr($data, 0, 8));
        if ($bottom != 0) {
            $data = $top / $bottom;
        } elseif ($top == 0) {
            $data = 0;
        } else {
            $data = $top . '/' . $bottom;
        }

        if ($tag == '0204') {
            // DigitalZoom
            $data .= 'x';
        }

        $result = $data;
    } elseif ($type == 'USHORT' || $type == 'SSHORT' || $type == 'ULONG' ||
              $type == 'SLONG' || $type == 'FLOAT' || $type == 'DOUBLE') {
        $data = bin2hex($data);
        $result['RAWDATA'] = $data;

        if ($tag == '0001') {
            // First chunk.
            $result['Bytes'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 0

            $result['Macro'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 1
            switch ($result['Macro']) {
            case 1: $result['Macro'] = 'Macro'; break;
            case 2: $result['Macro'] = 'Normal'; break;
            }

            $result['SelfTimer'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 2
            switch ($result['SelfTimer']) {
            case 0: $result['SelfTimer'] = 'Off'; break;
            }

            $result['Quality'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 3
            switch ($result['Quality']) {
            case 2: $result['Quality'] = 'Normal'; break;
            case 3: $result['Quality'] = 'Fine'; break;
            case 5: $result['Quality'] = 'Superfine'; break;
            }

            $result['Flash'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 4
            switch ($result['Flash']) {
            case 0: $result['Flash'] = 'Off'; break;
            case 1: $result['Flash'] = 'Auto'; break;
            case 2: $result['Flash'] = 'On'; break;
            case 3: $result['Flash'] = 'Red Eye Reduction'; break;
            case 4: $result['Flash'] = 'Slow Synchro'; break;
            case 5: $result['Flash'] = 'Auto + Red Eye Reduction'; break;
            case 6: $result['Flash'] = 'On + Red Eye Reduction'; break;
            case 16: $result['Flash'] = 'External Flash'; break;
            }

            $result['DriveMode'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 5
            switch ($result['DriveMode']) {
            case 0: $result['DriveMode'] = 'Single/Timer'; break;
            case 1: $result['DriveMode'] = 'Continuoue'; break;
            }

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 6

            $result['FocusMode'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 7
            switch ($result['FocusMode']) {
            case 0: $result['FocusMode'] = 'One-Shot'; break;
            case 1: $result['FocusMode'] = 'AI Servo'; break;
            case 2: $result['FocusMode'] = 'AI Focus'; break;
            case 3: $result['FocusMode'] = 'Manual Focus'; break;
            case 4: $result['FocusMode'] = 'Single'; break;
            case 5: $result['FocusMode'] = 'Continuous'; break;
            case 6: $result['FocusMode'] = 'Manual Focus'; break;
            }

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 8
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 9

            $result['ImageSize'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 10
            switch ($result['ImageSize']) {
            case 0: $result['ImageSize'] = 'Large'; break;
            case 1: $result['ImageSize'] = 'Medium'; break;
            case 2: $result['ImageSize'] = 'Small'; break;
            }

            $result['EasyShooting'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 11
            switch ($result['EasyShooting']) {
            case 0: $result['EasyShooting'] = 'Full Auto'; break;
            case 1: $result['EasyShooting'] = 'Manual'; break;
            case 2: $result['EasyShooting'] = 'Landscape'; break;
            case 3: $result['EasyShooting'] = 'Fast Shutter'; break;
            case 4: $result['EasyShooting'] = 'Slow Shutter'; break;
            case 5: $result['EasyShooting'] = 'Night'; break;
            case 6: $result['EasyShooting'] = 'Black & White'; break;
            case 7: $result['EasyShooting'] = 'Sepia'; break;
            case 8: $result['EasyShooting'] = 'Portrait'; break;
            case 9: $result['EasyShooting'] = 'Sport'; break;
            case 10: $result['EasyShooting'] = 'Macro/Close-Up'; break;
            case 11: $result['EasyShooting'] = 'Pan Focus'; break;
            }

            $result['DigitalZoom'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 12
            if ($result['DigitalZoom'] == 0) $result['DigitalZoom'] = 'No Digital Zoom';
            elseif ($result['DigitalZoom'] == 1) $result['DigitalZoom'] = '2x';
            elseif ($result['DigitalZoom'] == 2) $result['DigitalZoom'] = '4x';
            elseif ($result['DigitalZoom'] == 65535) $result['DigitalZoom'] = 'No Digital Zoom';

            $result['Contrast'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 13
            if ($result['Contrast'] == 0) $result['Contrast'] = 'Normal';
            elseif ($result['Contrast'] == 1) $result['Contrast'] = 'High';
            elseif ($result['Contrast'] == 65535) $result['Contrast'] = 'Low';

            $result['Saturation'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 14
            if ($result['Saturation'] == 0) $result['Saturation'] = 'Normal';
            elseif ($result['Saturation'] == 1) $result['Saturation'] = 'High';
            elseif ($result['Saturation'] == 65535) $result['Saturation'] = 'Low';

            $result['Sharpness'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 15
            if ($result['Sharpness'] == 0) $result['Sharpness'] = 'Normal';
            elseif ($result['Sharpness'] == 1) $result['Sharpness'] = 'High';
            elseif ($result['Sharpness'] == 65535) $result['Sharpness'] = 'Low';

            $result['ISO'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 16
            switch ($result['ISO']) {
            case 15: $result['ISO'] = 'Auto'; break;
            case 16: $result['ISO'] = '50'; break;
            case 17: $result['ISO'] = '100'; break;
            case 18: $result['ISO'] = '200'; break;
            case 19: $result['ISO'] = '400'; break;
            }

            $result['MeteringMode'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 17
            switch ($result['MeteringMode']) {
            case 3: $result['MeteringMode'] = 'Evaluative'; break;
            case 4: $result['MeteringMode'] = 'Partial'; break;
            case 5: $result['MeteringMode'] = 'Center-weighted'; break;
            }

            $result['FocusType'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 18
            if ($result['FocusType'] == 0) $result['FocusType'] = 'Manual';
            elseif ($result['FocusType'] == 1) $result['FocusType'] = 'Auto';
            elseif ($result['FocusType'] == 3) $result['FocusType'] = 'Close-up (Macro)';
            elseif ($result['FocusType'] == 8) $result['FocusType'] = 'Locked (Pan Mode)';

            $result['AFPointSelected'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 19
            if ($result['AFPointSelected'] == 12288) $result['AFPointSelected'] = 'Manual Focus';
            elseif ($result['AFPointSelected'] == 12289) $result['AFPointSelected'] = 'Auto Selected';
            elseif ($result['AFPointSelected'] == 12290) $result['AFPointSelected'] = 'Right';
            elseif ($result['AFPointSelected'] == 12291) $result['AFPointSelected'] = 'Center';
            elseif ($result['AFPointSelected'] == 12292) $result['AFPointSelected'] = 'Left';

            $result['ExposureMode'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 20
            switch ($result['ExposureMode']) {
            case 0: $result['ExposureMode'] = 'EasyShoot'; break;
            case 1: $result['ExposureMode'] = 'Program'; break;
            case 2: $result['ExposureMode'] = 'Tv'; break;
            case 3: $result['ExposureMode'] = 'Av'; break;
            case 4: $result['ExposureMode'] = 'Manual'; break;
            case 5: $result['ExposureMode'] = 'Auto-DEP'; break;
            }

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 21
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 22

            $result['LongFocalLength'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 23
            $result['ShortFocalLength'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 24
            $result['FocalUnits'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 25

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 26
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 27

            $result['FlashActivity'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 28
            if ($result['FlashActivity'] == 0) $result['FlashActivity'] = 'Flash Did Not Fire';
            elseif ($result['FlashActivity'] == 1) $result['FlashActivity'] = 'Flash Fired';

            $result['FlashDetails'] = str_pad(base_convert(intel2Moto(substr($data, $place, 4)), 16, 2), 16, '0', STR_PAD_LEFT); $place += 4; // 29
            $flashDetails = array();
            if (substr($result['FlashDetails'], 1, 1) == 1) $flashDetails[] = 'External E-TTL';
            if (substr($result['FlashDetails'], 2, 1) == 1) $flashDetails[] = 'Internal Flash';
            if (substr($result['FlashDetails'], 4, 1) == 1) $flashDetails[] = 'FP sync used';
            if (substr($result['FlashDetails'], 8, 1) == 1) $flashDetails[] = '2nd("rear")-curtin sync used';
            if (substr($result['FlashDetails'], 12, 1) == 1) $flashDetails[] = '1st curtin sync';
            $result['FlashDetails'] = implode(', ', $flashDetails);

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 30
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 31

            $result['FocusMode'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 32
        } elseif ($tag == '0004') {
            // Second chunk.
            $result['Bytes'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 0

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 1
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 2
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 3
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 4
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 5
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 6

            $result['WhiteBalance'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 7
            switch ($result['WhiteBalance']) {
            case 0: $result['WhiteBalance'] = 'Auto'; break;
            case 1: $result['WhiteBalance'] = 'Sunny'; break;
            case 2: $result['WhiteBalance'] = 'Cloudy'; break;
            case 3: $result['WhiteBalance'] = 'Tungsten'; break;
            case 4: $result['WhiteBalance'] = 'Flourescent'; break;
            case 5: $result['WhiteBalance'] = 'Flash'; break;
            case 6: $result['WhiteBalance'] = 'Custom'; break;
            }

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 8

            $result['SequenceNumber'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 9

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 10
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 11
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 12
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 13

            $result['AFPointUsed'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 14
            $result['FlashBias'] = hexdec(intel2Moto(substr($data, $place, 4))) . 'EV'; $place += 4; // 15

            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 16
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 17
            $unknown = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 18

            $result['SubjectDistance'] = hexdec(intel2Moto(substr($data, $place, 4))); $place += 4; // 19
        } elseif ($tag == '0008') {
            // image number
            if ($intel == 1) {
                $data = intel2Moto($data);
            }
            $data = hexdec($data);
            $result = round($data / 1000) . '-' . $data % 1000;
        } elseif ($tag == '000c') {
            // camera serial number
            if ($intel == 1) {
                $data = intel2Moto($data);
            }
            $data = hexdec($data);
            $result = '#' . bin2hex(substr($data, 0, 16))
                . substr($data, 16, 16);
        }
    } elseif ($type == 'UNDEFINED') {
    } else {
        $data = bin2hex($data);
        if ($intel == 1) {
            $data = intel2Moto($data);
        }
        $result = $data;
    }
}


/**
 * Cannon Special data section
 * Useful:
 *   http://www.burren.cx/david/canon.html
 *   http://www.ozhiker.com/electronics/pjmt/jpeg_info/canon_mn.html
 */
function parseCanon($block, &$result, $seek, $globalOffset)
{
    $place = 0; // current place

    if ($result['Endian'] == 'Intel') $intel = 1;
    else $intel = 0;

    $model = $result['IFD0']['Model'];

    // Get number of tags (2 bytes)
    $num = bin2hex(substr($block, $place, 2)); $place += 2;
    if ($intel == 1) $num = intel2Moto($num);
    $result['SubIFD']['MakerNote']['MakerNoteNumTags'] = hexdec($num);

    // Loop through all tags. Each field is 12 bytes.
    for ($i = 0; $i < hexdec($num); $i++) {
        // 2 byte tag
        $tag = bin2hex(substr($block, $place, 2)); $place += 2;
        if ($intel == 1) $tag = intel2Moto($tag);
        $tag_name = lookup_Canon_tag($tag);

        // 2 byte type
        $type = bin2hex(substr($block, $place, 2)); $place += 2;
        if ($intel == 1) $type = intel2Moto($type);
        lookup_type($type, $size);

        // 4 byte count of number of data units
        $count = bin2hex(substr($block, $place, 4)); $place += 4;
        if ($intel == 1) $count = intel2Moto($count);
        $bytesofdata = $size * hexdec($count);

        if ($bytesofdata <= 0) {
            // If this value is 0 or less then we have read all the
            // tags we can.
            return;
        }

        // 4 byte value of data or pointer to data.
        $value = substr($block, $place, 4); $place += 4;

        if ($bytesofdata <= 4) {
            $data = $value;
        } else {
            $value = bin2hex($value);
            if ($intel == 1) $value = intel2Moto($value);
            // Offsets are from TIFF header which is 12 bytes from the
            // start of the file.
            $v = fseek($seek, $globalOffset + hexdec($value));
            if ($v == 0) {
                $data = fread($seek, $bytesofdata);
            } elseif ($v == -1) {
                $result['Errors']++;
            }
        }

        formatCanonData($type, $tag, $intel, $data, $result['SubIFD']['MakerNote'][$tag_name]);
    }
}
