<?php
/**
 * Exifer
 * Extracts EXIF information from digital photos.
 *
 * $Horde: ansel/lib/Exif/fujifilm.php,v 1.7 2006/12/26 00:45:27 chuck Exp $
 *
 * Copyright 2003 Jake Olefsky
 * http://www.offsky.com/software/exif/index.php
 * jake@olefsky.com
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * http://www.gnu.org/copyleft/gpl.html
 */

/**
 * Looks up the name of the tag for the MakerNote (Depends on Manufacturer)
 *
 * @param string $tag  The tag code.
 *
 * @return string  The name for tag.
 */
function lookup_Fujifilm_tag($tag)
{
    switch ($tag) {
    case '0000': $tag = 'Version'; break;
    case '1000': $tag = 'Quality'; break;
    case '1001': $tag = 'Sharpness'; break;
    case '1002': $tag = 'WhiteBalance'; break;
    case '1003': $tag = 'Color'; break;
    case '1004': $tag = 'Tone'; break;
    case '1010': $tag = 'FlashMode'; break;
    case '1011': $tag = 'FlashStrength'; break;
    case '1020': $tag = 'Macro'; break;
    case '1021': $tag = 'FocusMode'; break;
    case '1030': $tag = 'SlowSync'; break;
    case '1031': $tag = 'PictureMode'; break;
    case '1032': $tag = 'Unknown'; break;
    case '1100': $tag = 'ContinuousTakingBracket'; break;
    case '1200': $tag = 'Unknown'; break;
    case '1300': $tag = 'BlurWarning'; break;
    case '1301': $tag = 'FocusWarning'; break;
    case '1302': $tag = 'AEWarning'; break;

    default: $tag = 'Unknown: ' . $tag;
    }

    return $tag;
}

/**
 * Formats Data for the data type.
 *
 * @param string $type
 * @param string $tag
 * @param string $data
 *
 * @return string  The data.
 */
function formatFujifilmData($type, $tag, $data)
{
    if ($type == 'ASCII') {
    } elseif ($type == 'URATIONAL' || $type == 'SRATIONAL') {
        $data = bin2hex($data);
        $data = intel2Moto($data);
        $top = hexdec(substr($data, 8, 8));
        $bottom = hexdec(substr($data, 0, 8));
        if ($bottom != 0) $data = $top / $bottom;
        elseif ($top == 0) $data = 0;
        else $data = $top . '/' . $bottom;

        if ($tag == '1011') {
            // FlashStrength
            $data .= ' EV';
        }
    } elseif ($type == 'USHORT' || $type == 'SSHORT' || $type == 'ULONG' ||
              $type == 'SLONG' || $type == 'FLOAT' || $type == 'DOUBLE') {
        $data = bin2hex($data);
        $data = intel2Moto($data);
        $data = hexdec($data);

        if ($tag == '1001') {
            // Sharpness
            switch ($data) {
            case 1:
            case 2: $data = 'Soft'; break;
            case 3: $data = 'Normal'; break;
            case 4:
            case 5: $data = 'Hard'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1002') {
            // WhiteBalance
            switch ($data) {
            case 0: $data = 'Auto'; break;
            case 256: $data = 'Daylight'; break;
            case 512: $data = 'Cloudy'; break;
            case 768: $data = 'DaylightColor-fluorescence'; break;
            case 769: $data = 'DaywhiteColor-fluorescence'; break;
            case 770: $data = 'White-fluorescence'; break;
            case 1024: $data = 'Incandenscense'; break;
            case 3840: $data = 'Custom'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1003') {
            // Color
            switch ($data) {
            case 0: $data = 'Chroma Saturation Normal(STD)'; break;
            case 256: $data = 'Chroma Saturation High'; break;
            case 512: $data = 'Chroma Saturation Low(ORG)'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1004') {
            // Tone
            switch ($data) {
            case 0: $data = 'Contrast Normal(STD)'; break;
            case 256: $data = 'Contrast High(HARD)'; break;
            case 512: $data = 'Contrast Low(ORG)'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1010') {
            // FlashMode
            switch ($data) {
            case 0: $data = 'Auto'; break;
            case 1: $data = 'On'; break;
            case 2: $data = 'Off'; break;
            case 3: $data = 'Red-Eye Reduction'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1020') {
            // Macro
            switch ($data) {
            case 0: $data = 'Off'; break;
            case 1: $data = 'On'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1021') {
            // FocusMode
            switch ($data) {
            case 0: $data = 'Auto'; break;
            case 1: $data = 'Manual'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1030') {
            // SlowSync
            switch ($data) {
            case 0: $data = 'Off'; break;
            case 1: $data = 'On'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1031') {
            // PictureMode
            switch ($data) {
            case 0: $data = 'Auto'; break;
            case 1: $data = 'Portrait'; break;
            case 2: $data = 'Landscape'; break;
            case 4: $data = 'Sports'; break;
            case 5: $data = 'Night'; break;
            case 6: $data = 'Program AE'; break;
            case 256: $data = 'Aperture Prority AE'; break;
            case 512: $data = 'Shutter Prority'; break;
            case 768: $data = 'Manual Exposure'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1100') {
            // ContinuousTakingBracket
            switch ($data) {
            case 0: $data = 'Off'; break;
            case 1: $data = 'On'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1300') {
            // BlurWarning
            switch ($data) {
            case 0: $data = 'No Warning'; break;
            case 1: $data = 'Warning'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1301') {
            // FocusWarning
            switch ($data) {
            case 0: $data = 'Auto Focus Good'; break;
            case 1: $data = 'Out of Focus'; break;
            default: $data = 'Unknown: ' . $data;
            }
        } elseif ($tag == '1302') {
            // AEWarning
            switch ($data) {
            case 0: $data = 'AE Good'; break;
            case 1: $data = 'Over Exposure'; break;
            default: $data = 'Unknown: ' . $data;
            }
        }
    } elseif ($type == 'UNDEFINED') {
    } else {
        $data = bin2hex($data);
        $data = intel2Moto($data);
    }

    return $data;
}

/**
 * Fujifilm Special data section
 */
function parseFujifilm($block, &$result)
{
    $model = $result['IFD0']['Model'];

    $place = 8; // current place
    $offset = 8;

    $num = bin2hex(substr($block, $place, 4)); $place += 4;
    $num = intel2Moto($num);
    $result['SubIFD']['MakerNote']['Offset'] = hexdec($num);

    // Get number of tags (2 bytes).
    $num = bin2hex(substr($block, $place, 2)); $place += 2;
    $num = intel2Moto($num);
    $result['SubIFD']['MakerNote']['MakerNoteNumTags'] = hexdec($num);

    // Loop through all tags. Each field is 12 bytes.
    for ($i = 0; $i < hexdec($num); $i++) {
        // 2 byte tag
        $tag = bin2hex(substr($block, $place, 2)); $place += 2;
        $tag = intel2Moto($tag);
        $tag_name = lookup_Fujifilm_tag($tag);

        // 2 byte type
        $type = bin2hex(substr($block, $place, 2)); $place += 2;
        $type = intel2Moto($type);
        lookup_type($type, $size);

        // 4 byte count of number of data units
        $count = bin2hex(substr($block, $place, 4)); $place += 4;
        $count = intel2Moto($count);
        $bytesofdata = $size * hexdec($count);

        // 4 byte value of data or pointer to data
        $value = substr($block, $place, 4); $place += 4;

        if ($bytesofdata <= 4) {
            $data = $value;
        } else {
            $value = bin2hex($value);
            $value = intel2Moto($value);
            $data = substr($block, hexdec($value) - $offset, $bytesofdata * 2);
        }

        $result['SubIFD']['MakerNote'][$tag_name] = formatFujifilmData($type, $tag, $data);
    }
}
