<?php
/**
 * Exifer
 * Extracts EXIF information from digital photos.
 *
 * $Horde: ansel/lib/Exif/nikon.php,v 1.8 2006/12/26 00:45:27 chuck Exp $
 *
 * Copyright 2003 Jake Olefsky
 * http://www.offsky.com/software/exif/index.php
 * jake@olefsky.com
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * http://www.gnu.org/copyleft/gpl.html
 */

/**
 * Looks up the name of the tag for the MakerNote (Depends on Manufacturer)
 *
 * @param string $tag  The tag code.
 * @param integer $model
 *
 * @return string  The name for tag.
 */
function lookup_Nikon_tag($tag, $model)
{
    switch ($model) {
    case 0:
        switch ($tag) {
        case '0003': $tag = 'Quality'; break;
        case '0004': $tag = 'ColorMode'; break;
        case '0005': $tag = 'ImageAdjustment'; break;
        case '0006': $tag = 'CCDSensitivity'; break;
        case '0007': $tag = 'WhiteBalance'; break;
        case '0008': $tag = 'Focus'; break;
        case '0009': $tag = 'Unknown2'; break;
        case '000a': $tag = 'DigitalZoom'; break;
        case '000b': $tag = 'Converter'; break;

        default: $tag = 'unknown:' . $tag;
        }

    case 1:
        switch ($tag) {
        case '0002': $tag = 'ISOSetting'; break;
        case '0003': $tag = 'ColorMode'; break;
        case '0004': $tag = 'Quality'; break;
        case '0005': $tag = 'Whitebalance'; break;
        case '0006': $tag = 'ImageSharpening'; break;
        case '0007': $tag = 'FocusMode'; break;
        case '0008': $tag = 'FlashSetting'; break;
        case '0009': $tag = 'FlashMode'; break;
        case '000b': $tag = 'WhiteBalanceFine'; break;
        case '000f': $tag = 'ISOSelection'; break;
        case '0013': $tag = 'ISOSelection2'; break;
        case '0080': $tag = 'ImageAdjustment'; break;
        case '0081': $tag = 'ToneCompensation'; break;
        case '0082': $tag = 'Adapter'; break;
        case '0083': $tag = 'LensType'; break;
        case '0084': $tag = 'LensInfo'; break;
        case '0085': $tag = 'ManualFocusDistance'; break;
        case '0086': $tag = 'DigitalZoom'; break;
        case '0087': $tag = 'FlashUsed'; break;
        case '0088': $tag = 'AFFocusPosition'; break;
        case '008d': $tag = 'ColorMode'; break;
        case '0090': $tag = 'LightType'; break;
        case '0094': $tag = 'Saturation'; break;
        case '0095': $tag = 'NoiseReduction'; break;
        case '0010': $tag = 'DataDump'; break;

        default: $tag = 'unknown:' . $tag; break;
        }
        break;
    }

    return $tag;
}

/**
 * Formats Data for the data type
 *
 * @param string $type
 * @param string $tag
 * @param integer|boolean $intel
 * @param string $data
 *
 * @return string  The data.
 */
function formatNikonData($type, $tag, $intel, $model, $data)
{
    if ($type == 'ASCII') {
    } elseif ($type == 'URATIONAL' || $type == 'SRATIONAL') {
        $data = bin2hex($data);
        if ($intel == 1) $data = intel2Moto($data);
        $top = hexdec(substr($data, 8, 8));
        $bottom = hexdec(substr($data, 0, 8));
        if ($bottom != 0) $data = $top / $bottom;
        elseif ($top == 0) $data = 0;
        else $data = $top . '/' . $bottom;

        if ($tag == '0085' && $model == 1) {
            // ManualFocusDistance
            $data .= ' m';
        }
        if ($tag == '0086' && $model == 1) {
            // DigitalZoom
            $data .= 'x';
        }
        if ($tag == '000a' && $model == 0) {
            // DigitalZoom
            $data .= 'x';
        }
    } elseif ($type == 'USHORT' || $type == 'SSHORT' || $type == 'ULONG' ||
              $type == 'SLONG' || $type == 'FLOAT' || $type == 'DOUBLE') {
        $data = bin2hex($data);
        if ($intel == 1) $data = intel2Moto($data);
        $data = hexdec($data);

        // Quality
        if ($tag == '0003' && $model == 0) {
            switch ($data) {
            case 1: $data = 'VGA Basic'; break;
            case 2: $data = 'VGA Normal'; break;
            case 3: $data = 'VGA Fine'; break;
            case 4: $data = 'SXGA Basic'; break;
            case 5: $data = 'SXGA Normal'; break;
            case 6: $data = 'SXGA Fine'; break;
            default: $data = 'Unknown: ' . $data;
            }
        }

        // Color
        if ($tag == '0004' && $model == 0) {
            switch ($data) {
            case 1: $data = 'Color'; break;
            case 2: $data = 'Monochrome'; break;
            default: $data = 'Unknown: ' . $data;
            }
        }

        // Image Adjustment
        if ($tag == '0005' && $model == 0) {
            switch ($data) {
            case 0: $data = 'Normal'; break;
            case 1: $data = 'Bright+'; break;
            case 2: $data = 'Bright-'; break;
            case 3: $data = 'Contrast+'; break;
            case 4: $data = 'Contrast-'; break;
            default: $data = 'Unknown: ' . $data; break;
            }
        }

        // CCD Sensitivity
        if ($tag == '0006' && $model == 0) {
            switch ($data) {
            case 0: $data = 'ISO-80'; break;
            case 2: $data = 'ISO-160'; break;
            case 4: $data = 'ISO-320'; break;
            case 5: $data = 'ISO-100'; break;
            default: $data = 'Unknown: ' . $data;
            }
        }

        // White Balance
        if ($tag == '0007' && $model == 0) {
            switch ($data) {
            case 0: $data = 'Auto'; break;
            case 1: $data = 'Preset'; break;
            case 2: $data = 'Daylight'; break;
            case 3: $data = 'Incandescense'; break;
            case 4: $data = 'Flourescence'; break;
            case 5: $data = 'Cloudy'; break;
            case 6: $data = 'SpeedLight'; break;
            default: $data = 'Unknown: ' . $data;
            }
        }

        // Converter
        if ($tag == '000b' && $model == 0) {
            switch ($data) {
            case 0: $data = 'None'; break;
            case 1: $data = 'Fisheye'; break;
            default: $data = 'Unknown: ' . $data;
            }
        }
    } elseif ($type == 'UNDEFINED') {
        // Unknown (Version?)
        if ($tag == '0001' && $model == 1) {
            $data /= 100;
        }
        // AF Focus Position
        if ($tag == '0088' && $model == 1) {
            $data = bin2hex($data);
            $data = str_replace(array('01',  '02',     '03',   '04',    '00'),
                                array('Top', 'Bottom', 'Left', 'Right', ''),
                                $data);
            if (!$data) {
                $data = 'Center';
            }
        }
    } else {
        $data = bin2hex($data);
        if ($intel == 1) {
            $data = intel2Moto($data);
        }

        // Lens Type
        if ($tag == '0083' && $model == 1) {
            $data = hexdec(substr($data, 0, 2));
            if ($data == 0) $data = 'AF non D';
            elseif ($data == 1) $data = 'Manual';
            elseif ($data == 2) $data = 'AF-D or AF-S';
            elseif ($data == 6) $data = 'AF-D G';
            elseif ($data == 10) $data = 'AF-D VR';
            else $data = 'Unknown: ' . $data;
        }

        // Flash type
        if ($tag == '0087' && $model == 1) {
            $data = hexdec(substr($data, 0, 2));
            if ($data == 0) $data = 'Did Not Fire';
            elseif($data == 4) $data = 'Unknown';
            elseif($data == 7) $data = 'External';
            elseif($data == 9) $data = 'On Camera';
            else $data = 'Unknown: ' . $data;
        }
    }

    return $data;
}


/**
 * Nikon Special data section
 */
function parseNikon($block, &$result)
{
    if ($result['Endian'] == 'Intel') $intel = 1;
    else $intel = 0;

    $model = $result['IFD0']['Model'];

    // These 6 models start with "Nikon".
    if ($model == "E700\0" || $model == "E800\0" || $model == "E900\0" ||
        $model == "E900S\0" || $model == "E910\0" || $model == "E950\0") {
        $place = 8; // current place
        $model = 0;

        // Get number of tags (2 bytes).
        $num = bin2hex(substr($block, $place, 2)); $place += 2;
        if ($intel == 1) $num = intel2Moto($num);
        $num = hexdec($num);
        $result['SubIFD']['MakerNote']['MakerNoteNumTags'] = $num;

        // Loop through all tags. Each field is 12 bytes.
        for ($i = 0; $i < $num; $i++) {
            // 2 byte tag
            $tag = bin2hex(substr($block, $place, 2)); $place += 2;
            if ($intel == 1) $tag = intel2Moto($tag);
            $tag_name = lookup_Nikon_tag($tag, $model);

            // 2 byte type
            $type = bin2hex(substr($block, $place, 2)); $place += 2;
            if ($intel == 1) $type = intel2Moto($type);
            lookup_type($type, $size);

            // 4 byte count of number of data units
            $count = bin2hex(substr($block, $place, 4)); $place += 4;
            if ($intel == 1) $count = intel2Moto($count);
            $bytesofdata = $size * hexdec($count);

            // 4 byte value of data or pointer to data
            $value = substr($block, $place, 4); $place += 4;

            // If tag is 0002 then it's the ASCII value which we know
            // is at 140, so calculate offset. THIS HACK ONLY WORKS
            // WITH EARLY NIKON MODELS.
            if ($tag == '0002') $offset = hexdec($value) - 140;
            if ($bytesofdata <= 4) {
                $data = $value;
            } else {
                $value = bin2hex($value);
                if ($intel == 1) $value = intel2Moto($value);
                $data = substr($block, hexdec($value) - $offset, $bytesofdata * 2);
            }

            $result['SubIFD']['MakerNote'][$tag_name] = formatNikonData($type, $tag, $intel, $model, $data);
        }
    } else {
        $place = 0; // current place
        $model = 1;

        $nikon = substr($block, $place, 8); $place += 8;
        $endian = substr($block, $place, 4); $place += 4;

        // 2 bytes of 0x002a
        $tag = bin2hex(substr($block, $place, 2)); $place += 2;

        // Then 4 bytes of offset to IFD0 (usually 8 which includes
        // all 8 bytes of TIFF header).
        $offset = bin2hex(substr($block, $place, 4)); $place += 4;
        if ($intel == 1) $offset = intel2Moto($offset);
        if (hexdec($offset) > 8) $place += $offset - 8;

        // Get number of tags (2 bytes).
        $num = bin2hex(substr($block, $place, 2)); $place += 2;
        if ($intel == 1) $num = intel2Moto($num);
        $num = hexdec($num);

        // Loop through all tags. Each field is 12 bytes.
        for ($i = 0; $i < $num; $i++) {
            // 2 byte tag
            $tag = bin2hex(substr($block, $place, 2)); $place += 2;
            if ($intel == 1) $tag = intel2Moto($tag);
            $tag_name = lookup_Nikon_tag($tag, $model);

            // 2 byte type
            $type = bin2hex(substr($block, $place, 2)); $place += 2;
            if ($intel == 1) $type = intel2Moto($type);
            lookup_type($type, $size);

            // 4 byte count of number of data units
            $count = bin2hex(substr($block, $place, 4)); $place += 4;
            if ($intel == 1) $count = intel2Moto($count);
            $bytesofdata = $size * hexdec($count);

            // 4 byte value of data or pointer to data
            $value = substr($block, $place, 4); $place += 4;

            if ($bytesofdata <= 4) {
                $data = $value;
            } else {
                $value = bin2hex($value);
                if ($intel == 1) $value = intel2Moto($value);
                $data = substr($block, hexdec($value) + hexdec($offset) + 1, $bytesofdata);
            }

            $result['SubIFD']['MakerNote'][$tag_name] = formatNikonData($type, $tag, $intel, $model, $data);
        }
    }
}
