<?php
/**
 * Exifer
 * Extracts EXIF information from digital photos.
 *
 * $Horde: ansel/lib/Exif/sanyo.php,v 1.6 2006/12/26 00:45:27 chuck Exp $
 *
 * Copyright 2003 Jake Olefsky
 * http://www.offsky.com/software/exif/index.php
 * jake@olefsky.com
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 * http://www.gnu.org/copyleft/gpl.html
 */

/**
 * Looks up the name of the tag for the MakerNote (Depends on Manufacturer)
 *
 * @param string $tag  The tag code.
 *
 * @return string  The name for tag.
 */
function lookup_Sanyo_tag($tag)
{
    switch ($tag) {
    case '0200': $tag = 'SpecialMode'; break;
    case '0201': $tag = 'Quality'; break;
    case '0202': $tag = 'Macro'; break;
    case '0203': $tag = 'Unknown'; break;
    case '0204': $tag = 'DigiZoom'; break;
    case '0f00': $tag = 'DataDump'; break;
    default: $tag = 'unknown:'.$tag;
    }

    return $tag;
}

/**
 * Formats Data for the data type
 *
 * @param string $type
 * @param string $tag
 * @param integer|boolean $intel
 * @param string $data
 *
 * @return string  The data.
 */
function formatSanyoData($type, $tag, $intel, $data)
{
    if ($type == 'ASCII') {
    } elseif ($type == 'URATIONAL' || $type == 'SRATIONAL') {
        $data = bin2hex($data);
        if ($intel == 1) $data = intel2Moto($data);
        $top = hexdec(substr($data, 8, 8));
        $bottom = hexdec(substr($data, 0, 8));
        if ($bottom != 0) $data = $top / $bottom;
        elseif ($top == 0) $data = 0;
        else $data = $top . '/' . $bottom;
    } elseif ($type == 'USHORT' || $type == 'SSHORT' || $type == 'ULONG' ||
              $type == 'SLONG' || $type == 'FLOAT' || $type == 'DOUBLE') {
        $data = bin2hex($data);
        if ($intel == 1) $data = intel2Moto($data);
        $data = hexdec($data);

        // SpecialMode
        if ($tag == '0200') {
            if ($data == 0) $data = 'Normal';
            else $data = 'Unknown: ' . $data;
        }

        // Quality
        if ($tag == '0201') {
            if ($data == 2) $data = 'High';
            else $data = 'Unknown: ' . $data;
        }

        // Macro
        if ($tag == '0202') {
            if ($data == 0) $data = 'Normal';
            else $data = 'Unknown: ' . $data;
        }
    } elseif ($type == 'UNDEFINED') {
    } else {
        $data = bin2hex($data);
        if ($intel == 1) $data = intel2Moto($data);
    }

    return $data;
}

/**
 * Sanyo special data section
 */
function parseSanyo($block, &$result, $seek, $globalOffset)
{
    $intel = ($result['Endian'] == 'Intel') ? 1 : 0;

    $model = $result['IFD0']['Model'];

    $place = 8; // current place
    $offset = 8;

    // Get number of tags (2 bytes)
    $num = bin2hex(substr($block, $place, 2)); $place += 2;
    if ($intel == 1) $num = intel2Moto($num);
    $result['SubIFD']['MakerNote']['MakerNoteNumTags'] = hexdec($num);

    // Loop through all tags. Each field is 12 bytes.
    for ($i = 0; $i < hexdec($num); $i++) {
        // 2 byte tag
        $tag = bin2hex(substr($block, $place, 2)); $place += 2;
        if ($intel == 1) $tag = intel2Moto($tag);
        $tag_name = lookup_Sanyo_tag($tag);

        // 2 byte type
        $type = bin2hex(substr($block, $place, 2)); $place += 2;
        if ($intel == 1) $type = intel2Moto($type);
        lookup_type($type, $size);

        // 4 byte count of number of data units
        $count = bin2hex(substr($block, $place, 4)); $place += 4;
        if ($intel == 1) $count = intel2Moto($count);
        $bytesofdata = $size * hexdec($count);

        // 4 byte value of data or pointer to data
        $value = substr($block, $place, 4); $place += 4;

        if ($bytesofdata <= 4) {
            $data = $value;
        } else {
            $value = bin2hex($value);
            if ($intel == 1) $value = intel2Moto($value);
            // Offsets are from TIFF header which is 12 bytes from the
            // start of the file.
            $v = fseek($seek, $globalOffset + hexdec($value));
            if ($v == 0) {
                $data = fread($seek, $bytesofdata);
            } elseif ($v == -1) {
                $result['Errors']++;
            }
        }

        $result['SubIFD']['MakerNote'][$tag_name] = formatSanyoData($type, $tag, $intel, $data);
    }
}
