<?php
/**
 * @package Ansel
 *
 * $Horde: ansel/lib/Views/Gallery.php,v 1.5 2006/12/02 07:13:55 chuck Exp $
 */

/** Ansel_View_Abstract */
require_once ANSEL_BASE . '/lib/Views/Abstract.php';

/** Horde_UI_Pager */
require_once 'Horde/UI/Pager.php';

/** Text_Filter */
require_once 'Horde/Text/Filter.php';

/** Variables */
require_once 'Horde/Variables.php';

/**
 * The Ansel_View_Gallery:: class wraps display of individual images.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @package Ansel
 */
class Ansel_View_Gallery extends Ansel_View_Abstract {

    /**
     */
    function Ansel_View_Gallery()
    {
        $this->gallery = $this->getGallery();
    }

    /**
     */
    function getTitle()
    {
        if (is_a($this->gallery, 'PEAR_Error')) {
            return $this->gallery->getMessage();
        }
        return $this->gallery->get('name');
    }

    /**
     */
    function html()
    {
        global $conf, $prefs, $registry;

        if (is_a($this->gallery, 'PEAR_Error')) {
            echo htmlspecialchars($this->gallery->getMessage());
            return;
        }

        $galleryId = $this->gallery->getId();
        $page = Util::getFormData('page', 0);
        $gpage = Util::getFormData('gpage', 0);
        $numimages = $this->gallery->count();
        $perpage = min($prefs->getValue('imagesperpage'), $conf['thumbnail']['perpage']);
        $pagestart = ($page * $perpage) + 1;
        $pageend = min($numimages, $pagestart + $perpage - 1);
        $images = $this->gallery->getImages($page * $perpage, $perpage);
        $viewurl = Util::addParameter('view.php', 'gallery', $galleryId);
        $galleries_perpage = $prefs->getValue('galleriesperpage');
        $vars = Variables::getDefaultVariables();
        $pager = new Horde_UI_Pager('page', $vars, array('num' => $numimages,
                                                         'url' => $viewurl,
                                                         'perpage' => $perpage));

        /* Get any subgalleries. */
        $num_galleries = Ansel::countGalleries(PERMS_SHOW, false, $this->gallery->getName(), false);
        if ($num_galleries) {
            $min = min($num_galleries, $gpage * $galleries_perpage);
            $max = min($num_galleries, $min + $galleries_perpage);

            $gallery_pager = new Horde_UI_Pager('gpage', $vars, array('num' => $num_galleries,
                                                                      'url' => $viewurl,
                                                                      'perpage' => $galleries_perpage));

            require ANSEL_TEMPLATES . '/view/subgallery_header.inc';
            require ANSEL_TEMPLATES . '/list/gallery_headers.inc';

            $galleries = Ansel::listGalleries(PERMS_SHOW, false, $this->gallery->getId(), false, $min, $galleries_perpage);
            foreach ($galleries as $gallery) {
                $viewurl = Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $gallery->getId()));
                require ANSEL_TEMPLATES . '/list/gallery_summaries.inc';
            }

            require ANSEL_TEMPLATES . '/list/gallery_footers.inc';
        }

        $uploadurl = Util::addParameter('img/upload.php', array('gallery' => $galleryId,
                                                                'page' => $page));
        $imgurl = Util::addParameter('image.php', array('gallery' => $galleryId,
                                                        'page' => $page));
        $captionurl = Util::addParameter('gallery/captions.php', 'gallery', $galleryId);
        $galleryurl = Util::addParameter('gallery.php', 'gallery', $galleryId);
        $viewurl = Util::addParameter('view.php', array('gallery' => $galleryId,
                                                        'page' => $page,
                                                        'view' => 'Image'));
        $sorturl = Util::addParameter('gallery/sort.php', 'gallery', $galleryId);

        $option_edit = $this->gallery->hasPermission(Auth::getAuth(), PERMS_EDIT);
        $option_select = $option_delete = $this->gallery->hasPermission(Auth::getAuth(), PERMS_DELETE);
        $option_move = ($option_delete && Ansel::countGalleries(PERMS_EDIT));

        include ANSEL_TEMPLATES . '/view/gallery.inc';
    }

    /**
     * JSON representation of the gallery.
     */
    function json()
    {
        $json = array();
        $images = $this->gallery->getImages();
        foreach ($images as $image) {
            $dimensions = $image->getDimensions('screen');
            if (is_a($dimensions, 'PEAR_Error')) {
                $dimensions = array('height' => $GLOBALS['conf']['screen']['height'],
                                    'width' => $GLOBALS['conf']['screen']['width']);
            }

            // Source, Width, Height, Name, Caption.
            $json[] = array(Ansel::getImageUrl($image->id, 'screen'),
                            $dimensions['width'],
                            $dimensions['height'],
                            htmlspecialchars($image->filename),
                            Text_Filter::filter($image->caption, 'text2html', array('parselevel' => TEXT_HTML_MICRO)));
        }

        require_once 'Horde/Serialize.php';
        echo Horde_Serialize::serialize($json, SERIALIZE_JSON, NLS::getCharset());
    }

}
