<?php
/**
 * @package Ansel
 *
 * $Horde: ansel/lib/Views/Image.php,v 1.4 2007/03/17 05:05:30 chuck Exp $
 */

/** Ansel_View_Abstract */
require_once ANSEL_BASE . '/lib/Views/Abstract.php';

/** Text_Filter */
require_once 'Horde/Text/Filter.php';

/**
 * The Ansel_View_Image:: class wraps display of individual images.
 *
 * $Horde: ansel/lib/Views/Image.php,v 1.4 2007/03/17 05:05:30 chuck Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @package Ansel
 */
class Ansel_View_Image extends Ansel_View_Abstract {

    var $gallery;
    var $image;

    function Ansel_View_Image()
    {
        $this->gallery = $this->getGallery();
        $image_id = Util::getFormData('image');

        /* If the requested image doesn't exist, display an error
         * message. */
        $this->image = $GLOBALS['ansel_shares']->getImage($image_id);

        Horde::addScriptFile('prototype.js', 'horde', true);
        Horde::addScriptFile('effects.js', 'ansel', true);
        Horde::addScriptFile('slideshow.js', 'ansel', true);
        Horde::addScriptFile('stripe.js', 'horde', true);
    }

    function getTitle()
    {
        return $this->image->filename;
    }

    function html()
    {
        global $browser, $conf, $prefs, $registry;

        $page = Util::getFormData('page', 0);
        $galleryId = $this->gallery->getId();
        $imageId = $this->image->id;

        /* Get comments before any output in sent. */
        if (($conf['comments']['allow'] == 'all' || ($conf['comments']['allow'] == 'authenticated' && Auth::getAuth())) &&
            $registry->hasMethod('forums/doComments')) {
            $comments = $registry->call('forums/doComments', array('ansel', $imageId, 'commentCallback'));
            if (is_a($comments, 'PEAR_Error')) {
                Horde::logMessage($comments, __FILE__, __LINE__, PEAR_LOG_DEBUG);
                $comments = array();
            }
        } else {
            $comments = array();
        }

        /* Find the next and previous images. */
        $imageList = $this->gallery->listImages();
        $revList = array_flip($imageList);
        $imageIndex = $revList[$imageId];
        if (isset($imageList[$imageIndex + 1])) {
            $next = $imageList[$imageIndex + 1];
        } else {
            $next = $imageList[0];
        }
        if (isset($imageList[$imageIndex - 1])) {
            $prev = $imageList[$imageIndex - 1];
        } else {
            $prev = $imageList[count($imageList) - 1];
        }

        /* Figure out what page the next & prev images will be on. */
        $perpage = $prefs->getValue('imagesperpage');
        $pagestart = $page * $perpage;
        $pageend = min(count($imageList), $pagestart + $perpage - 1);
        $page_next = $page;
        if ($revList[$imageId] + 1 > $pageend) {
            $page_next++;
        }
        $page_prev = $page;
        if ($revList[$imageId] - 1 < $pagestart) {
            $page_prev--;
        }

        $ecardurl = Util::addParameter('img/ecard.php', array('gallery' => $galleryId, 'image' => $imageId));
        $galleryurl = Util::addParameter('view.php', array('gallery' => $galleryId, 'page' => $page));
        $imageurl = Util::addParameter('image.php', array('gallery' => $galleryId, 'image' => $imageId, 'page' => $page));

        require ANSEL_TEMPLATES . '/view/image.inc';

        if (!empty($comments['threads'])) {
            echo '<br />' . $comments['threads'];
        }
        if (!empty($comments['comments'])) {
            echo '<br />' . $comments['comments'];
        }
    }

}
