<?php
/**
 * Ansel external API interface.
 *
 * $Horde: ansel/lib/api.php,v 1.43 2007/02/26 07:03:18 slusarz Exp $
 *
 * This file defines Ansel's external API interface. Other
 * applications can interact with Ansel through this API.
 *
 * @package Ansel
 */

$_services['commentCallback'] = array(
    'args' => array('image_id' => 'string'),
    'type' => 'string'
);

$_services['hasComments'] = array(
    'args' => array(),
    'type' => 'boolean'
);

$_services['saveImage'] = array(
    'args' => array('app'          => 'string',
                    'gallery_id'   => 'string',
                    'image'        => '{urn:horde}hashHash',
                    'default'      => 'boolean',
                    'gallery_data' => '{urn:horde}hashHash'),
    'type' => '{urn:horde}stringArray'
);

$_services['createGallery'] = array(
    'args' => array('app'        => 'string',
                    'galleryId'  => 'string',
                    'attributes' => '{urn:horde}hashHash',
                    'perm'       => 'object'),
    'type' => 'int'
);

$_services['getGalleryId'] = array(
    'args' => array('app'          => 'string',
                    'gallery_name' => 'string'),
    'type' => 'string'
);

$_services['removeImage'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string',
                    'image_id'   => 'string'),
    'type' => 'int'
);

$_services['removeGallery'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string'),
    'type' => 'int'
);

$_services['getImageUrl'] = array(
    'args' => array('app'        => 'string',
                    'image_id'   => 'string',
                    'view'       => 'string',
                    'full'       => 'boolean'),
    'type' => 'string'
);

$_services['count'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string'),
    'type' => 'int'
);

$_services['getDefaultImage'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string'),
    'type' => 'string'
);

$_services['listGalleries'] = array(
    'args' => array('app'        => 'string',
                    'perm'       => 'integer',
                    'parent'     => 'string',
                    'allLevels'  => 'string',
                    'from'       => 'integer',
                    'count'      => 'integer'),
    'type' => 'string'
);

$_services['selectGalleries'] = array(
    'args' => array('app'        => 'string',
                    'perm'       => 'integer',
                    'parent'     => 'string',
                    'allLevels'  => 'string',
                    'from'       => 'integer',
                    'count'      => 'integer'),
    'type' => 'string'
);

$_services['listImages'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string',
                    'perm'       => 'integer',
                    'view'       => 'string',
                    'full'       => 'boolean',
                    'from'       => 'integer',
                    'count'      => 'integer'),
    'type' => 'string'
);

$_services['countGalleries'] = array(
    'args' => array('app'        => 'string',
                    'perm'       => 'string',
                    'attributes' => '{urn:horde}hash',
                    'parent'     => 'string',
                    'allLevels'  => 'boolean'),
    'type' => 'int'
);

/**
 * Callback for Agora comments.
 */
function _ansel_commentCallback($image_id)
{
    if (!$GLOBALS['conf']['comments']['allow']) {
        return false;
    }

    require_once dirname(__FILE__) . '/base.php';

    $image = &$GLOBALS['ansel_shares']->getImage($image_id);
    if (!$image || is_a($image, 'PEAR_Error')) {
        return false;
    }

    return $image->filename;
}

function _ansel_hasComments()
{
    return !empty($GLOBALS['conf']['comments']['allow']);
}

/**
 * Stores an image in a gallery and returns gallery and image data.
 *
 * @param integer $app         Application used if null then use default.
 * @param integer $gallery_id  The gallery id to add the image to.
 * @param array $image         Image data array.  This can either be the return
 *                             from Horde_Form_Type_image:: or an array with
 *                             the following four fields:
 *                             'filename', 'description', 'data', 'type'
 * @param integer $default     Set this image as default in the gallery?
 * @param array $gallery_data  Gallery parameters (like image dimensions).
 *
 * @return mixed  False if error, or id of an array of image/gallery data.
 */
function _ansel_saveImage($app = null, $gallery_id, $image, $default = false,
                          $gallery_data = array())
{
    require_once dirname(__FILE__) . '/base.php';

    $image_data = null;

    /* If no app is given use Ansel's own gallery which is initialized
     * in base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    if (isset($image['filename']) &&
        isset($image['description']) &&
        isset($image['data']) &&
        isset($image['type'])) {
        $image_data = array('image_filename' => $image['filename'],
                            'image_caption' => $image['description'],
                            'image_type' => $image['type'],
                            'data' => $image['data']);
    }

    if (is_null($image_data) && getimagesize($image['file']) === false) {
        return PEAR::raiseError(_("The file you uploaded does not appear to be a valid image."));
    } elseif ($GLOBALS['ansel_shares']->exists($GLOBALS['ansel_shares']->getName($gallery_id))) {
        $gallery = $GLOBALS['ansel_shares']->getGallery($gallery_id);
        if (is_a($gallery, 'PEAR_Error')) {
            return $gallery;
        }
        if (!empty($gallery_data)) {
            foreach ($gallery_data as $key => $value) {
                $gallery->set($key, $value);
            }
            $gallery->save();
        }
    } else {
        return PEAR::raiseError(_("A gallery to add this image to is required."));
    }


    if (is_null($image_data)) {
        $image_data = array(
            'image_filename' => $image['name'],
            'image_caption' => $image['name'],
            'image_type' => $image['name']['type'],
            'data' => file_get_contents($image['file']),
        );
    }

    $image_id = $gallery->addImage($image_data);
    if (is_a($image_id, 'PEAR_Error')) {
        return $image_id;
    }

    $image = &$GLOBALS['ansel_shares']->getImage($image_id);
    if (is_a($image, 'PEAR_Error')) {
        return $image;
    }

    $image->createView('thumb');
    $image->createView('screen');

    if ($default) {
        $gallery->set('default', $image_id, true);
        $gallery->set('default-type', 'manual', true);
        $gallery->save();
    }

    return array('image-id'   => (int)$image_id,
                 'gallery-id' => (int)$gallery->getId(),
                 'num-images' => (int)$gallery->count());
}

/**
 * Removes an image from a gallery.
 *
 * @param string $app         Application used if null then use default.
 * @param integer $gallery_id The id of gallery.
 * @param string $image_id    The id of image to remove.
 */
function _ansel_removeImage($app = null, $gallery_id, $image_id)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    $image = &$GLOBALS['ansel_shares']->getImage($image_id);
    return $GLOBALS['ansel_shares']->removeImage($image);
}

/**
 * Add a new gallery to any arbitrary application's Ansel_Shares.
 *
 * @param string $app            The application to use if not default.
 * @param string $gallery_name   The unique gallery name to use if ansel's
 *                               default naming is not desirable.
 * @param array $attributes      The gallery attributes.
 * @param object Perms $perm     The permissions for the gallery if the
 *                               defaults are not desirable.
 *
 * @return mixed  The gallery id of the new gallery | Pear_Error
 */
function _ansel_createGallery($app = null, $gallery_name = '', $attributes = array(),
                            $perm = null)
{

    require_once dirname(__FILE__) . '/base.php';

    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    if ($gallery_name == '') {
        $gallery_name = md5(time());
    }

    $gallery = Ansel::createGallery($gallery_name, $attributes, $perm);
    if (is_a($gallery, 'PEAR_Error')) {
        return $gallery;
    }
    return $gallery->getId();
}

/**
 * Removes a gallery and its images.
 *
 * @param string $app         Application used if null then use default.
 * @param integer $gallery_id  The id of gallery.
 */
function _ansel_removeGallery($app = null, $gallery_id)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    $gallery = $GLOBALS['ansel_shares']->getGallery($gallery_id);
    if (is_a($gallery, 'PEAR_Error') ||
        !$gallery->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
        return PEAR::raiseError(sprintf(_("Access denied deleting gallery %s."),
                                        $gallery->get('name')));
    } else {
        $imageList = $gallery->listImages();
        $result = $GLOBALS['ansel_shares']->removeShare($gallery);
        if (!is_a($result, 'PEAR_Error')) {
            if ($imageList) {
                foreach ($imageList as $id) {
                    $gallery->removeImage($id);
                }
            }
            return true;
        } else {
            return PEAR::raiseError(sprintf(_("There was a problem deleting %s: %s"),
                                            $gallery->get('name'),
                                            $result->getMessage()));
        }
    }
}

/**
 * Get the datatree_id for a gallery.
 *
 * @param string $app           Application used if not default.
 * @param string $gallery_name  The name of the gallery share.
 *
 * @return mixed  The datatree_id | PEAR_Error
 */
function _ansel_getGalleryId($app = null, $gallery_name)
{
    require_once dirname(__FILE__) . '/base.php';

    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    return $GLOBALS['ansel_shares']->getIdFromName($gallery_name);
}

/**
 * Returns the number of images in a gallery.
 *
 * @param integer $app        Application used; if null then use default.
 * @param integer $gallery_id The id of gallery or image.
 *
 * @return integer  The number of images in the gallery.
 */
function _ansel_count($app = null, $gallery_id = null)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    $gallery = &$GLOBALS['ansel_shares']->getGallery($gallery_id);
    if (is_a($gallery, 'PEAR_Error')) {
        return 0;
    } else {
        return (int)$gallery->count();
    }
}

/**
 * Returns the default image id of a gallery.
 *
 * @param string $app          Application used.
 * @param integer $gallery_id  The id of gallery or image.
 *
 * @return integer  The default image id.
 */
function _ansel_getDefaultImage($app = null, $gallery_id)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    $gallery = $GLOBALS['ansel_shares']->getGallery($gallery_id);
    if (is_a($gallery, 'PEAR_Error')) {
        return $gallery;
    } else {
        return $gallery->getDefaultImage();
    }
}

/**
 * Returns image URL.
 *
 * @param integer $app       Application used.
 * @param integer $image_id  The id of the image.
 * @param string $view       The view ('screen', 'thumb', or 'full') to show.
 * @param boolean $full      Return a path that includes the server name?
 *
 * @return string  The image path.
 */
function _ansel_getImageUrl($app = null, $image_id, $view = 'screen', $full = false)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    return Ansel::getImageUrl($image_id, $view, $full);
}

/**
 * Returns a list of all galleries.
 *
 * @param string $app         Horde application used.
 * @param integer $perm       The level of permissions to require for a gallery
 *                            to be returned.
 * @param string $parent      The parent share to start searching from. This
 *                            should be either a gallery id or DATATREE_ROOT.
 * @param boolean $allLevels  Return all levels, or just the direct children of
 *                            $parent?
 * @param integer $from       The gallery to start listing at.
 * @param integer $count      The number of galleries to return.
 *
 * @return array  An array of Datatree_Object_Share objects..
 */
function _ansel_listGalleries($app = null, $perm = PERMS_SHOW,
                              $parent = DATATREE_ROOT,
                              $allLevels = true, $from = 0, $count = 0)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    return Ansel::listGalleries($perm, null, $parent, $allLevels, $from, $count);
}

/**
 * Returns a 'select' menu from the list of galleries created by
 * _ansel_listGalleries().
 *
 * @see _ansel_listGalleries() for parameters.
 */
function _ansel_selectGalleries($app = null, $perm = PERMS_SHOW,
                                $parent = DATATREE_ROOT,
                                $allLevels = true, $from = 0, $count = 0)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    return Ansel::selectGalleries(null, $perm, null, $parent, $allLevels, $from, $count);
}

/**
 * Returns a list of all images in a gallery.
 *
 * The return has the URL because in a lot of cases you'll want the url
 * also. Using api call getImageURL results in a lot of overhead when
 * e.g. generating a select list.
 *
 * @param string $app         Horde application used.
 * @param integer $gallery_id  Gallery to get images from.
 * @param integer $perm       The level of permissions to require for a
 *                            gallery to return it.
 * @param string $view        Viewsize to generate URLs for.
 * @param boolean $full       Return URL of full image.
 * @param integer $from       Start image
 * @param integer $count      End image.
 *
 * @return array  Two dimensional array with image names ids (key) and urls.
 */
function _ansel_listImages($app = null, $gallery_id = '', $perm = PERMS_SHOW,
                           $view = 'screen', $full = false, $from = 0, $count = 0)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    } else {
        $app = 'ansel';
    }

    /* Determine the default gallery when none is given. The first gallery in
       the list is the default gallery. */
    if ($gallery_id == '') {
        $galleries = Ansel::listGalleries($perm);
        if (!count($galleries)) {
            return array();
        }
        $keys = array_keys($galleries);
        $gallery_names = array_keys($galleries[$keys[0]]['galleries']);
        $gallery_id = $gallery_names[0];
    }

    $gallery = $GLOBALS['ansel_shares']->getGallery($gallery_id);
    if (is_a($gallery, 'PEAR_Error')) {
        return $gallery;
    }

    $images = $gallery->listImages();
    if (is_a($images, 'PEAR_Error')) {
        return $images;
    }

    $counter = 0;
    $imagelist = array();
    foreach ($images as $id) {
        $image = $GLOBALS['ansel_shares']->getImage($id);
        $imagelist[$id]['name'] = $image->filename;
        $imagelist[$id]['url'] = Ansel::getImageUrl($id, $view, $full);
        $imagelist[$id]['caption'] = $image->caption;
        if (!is_null($app) && empty($GLOBALS['conf']['vfs']['direct'])) {
            $imagelist[$id]['url'] = Util::addParameter($imagelist[$id]['url'], 'app', $app);
        }
    }
    return $imagelist;
}


/**
 * Counts the number of galleries.
 *
 * @param string $app         Horde application used.
 * @param integer $perm       The level of permissions to require for a gallery
 *                            to return it.
 * @param mixed $attributes   Restrict the galleries counted to those matching
 *                            $attributes. An array of attribute/value pairs or
 *                            a gallery owner username.
 * @param string $parent      The parent share to start searching at.
 * @param boolean $allLevels  Return all levels, or just the direct children of
 *                            $parent?
 *
 * @return integer  Returns the number of matching galleries.
 */
function _ansel_countGalleries($app = null, $perm = PERMS_SHOW, $attributes = null,
                               $parent = DATATREE_ROOT, $allLevels = true)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized
     * in base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = new Ansel_Shares($app);
    }

    return Ansel::countGalleries($perm, $attributes, $parent, $allLevels);
}
