<?php
/**
 * $Horde: ansel/list.php,v 1.66 2007/01/02 12:46:46 jan Exp $
 *
 * Copyright 2001-2007 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author Chuck Hagenbuch <chuck@horde.org>
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';

// If we aren't supplied with a page number, default to page 0.
$gpage = Util::getFormData('gpage', 0);
$galleries_perpage = $prefs->getValue('galleriesperpage');

// Check for grouping.
$groupby = Util::getFormData('groupby', $prefs->getValue('groupby'));
$owner = Util::getFormData('owner');
$special = Util::getFormData('special');
$category = Util::getFormData('category');

if (!$owner && !$category && !$special && $groupby != 'none' ) {
    header('Location: ' . Horde::applicationUrl(Util::addParameter('group.php', 'groupby', $groupby), true));
    exit;
}

// Set list filter/title
if ($owner) {
    $filter = $owner;

    if ($owner == Auth::getAuth()) {
        $list_title = _("My Galleries");
    } elseif ($owner) {
        require_once 'Horde/Identity.php';
        $identity = &Identity::singleton('none', $owner);
        $fullname = $identity->getValue('fullname');
        if (!$fullname) {
            $fullname = $owner;
        }
        $list_title = sprintf(_("%s's Galleries"), $fullname);
    }
} elseif ($category || ($groupby == 'category' && $special)) {
    if ($special == 'unfiled') {
        $list_title = sprintf(_("Galleries in category %s"), _("Unfiled"));
        $filter = array('category' => '');
    } else {
        $list_title = sprintf(_("Galleries in category %s"), $category);
        $filter = array('category' => $category);
    }
} else {
    $filter = null;
    $list_title = _("Gallery List");
}

$num_galleries = Ansel::countGalleries(PERMS_SHOW, $filter, DATATREE_ROOT, false);
if ($num_galleries == 0) {
    $notification->push(_("There are no photo galleries available."), 'horde.message');
    $gallerylist = array();
} else {
    $gallerylist = Ansel::listGalleries(PERMS_SHOW, $filter, DATATREE_ROOT, false, $gpage * $galleries_perpage, $galleries_perpage);
}

require_once 'Horde/UI/Pager.php';
require_once 'Horde/Variables.php';
$vars = Variables::getDefaultVariables();
$gallery_pager = new Horde_UI_Pager('gpage', $vars, array('num' => $num_galleries,
                                                          'url' => Util::addParameter('list.php', array('owner' => $owner, 'category' => $category)),
                                                          'perpage' => $galleries_perpage));

require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';

if ($num_galleries) {
    $min = $gpage * $galleries_perpage;
    $max = $min + $galleries_perpage;
    if ($max > $num_galleries) {
        $max = $num_galleries - $min;
    }
    $start = $min + 1;
    $end = min($num_galleries, $min + $galleries_perpage);

    if ($owner) {
        $refresh_link = Util::addParameter('list.php', array('groupby' => $groupby,
                                                             'owner' => $owner,
                                                             'gpage' => $gpage));
    } else {
        $refresh_link = 'list.php';
    }

    require ANSEL_TEMPLATES . '/list/header.inc';
    $count = 0;
    foreach ($gallerylist as $galleryId => $gallery) {
        $viewurl = Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $gallery->getId()));
        $galleryurl = Util::addParameter('gallery.php', 'gallery', $gallery->getId());
        $ownerurl = Horde::applicationUrl(Util::addParameter('list.php', 'owner', $gallery->get('owner')));
        require ANSEL_TEMPLATES . '/list/images.inc';
    }
    while ($count++ % $prefs->getValue('imagesperrow')) {
        require ANSEL_TEMPLATES . '/list/filler.inc';
    }
    require ANSEL_TEMPLATES . '/list/footer.inc';
    require ANSEL_TEMPLATES . '/list/pager.inc';
}

$title = _("Gallery List");
require $registry->get('templates', 'horde') . '/common-footer.inc';
