#!/usr/local/bin/php
<?php
/**
 * $Horde: ansel/scripts/upgrades/2006-11-27_datatree_to_sql_renames.php,v 1.3 2006/12/03 04:27:51 chuck Exp $
 *
 * This script renames files in the VFS backend and in gallery
 * "default" image settings to match the new SQL ids. It's a very
 * good idea to back up your VFS backend before running this script.
 */

@define('AUTH_HANDLER', true);
@define('ANSEL_BASE', dirname(__FILE__) . '/../..');
@define('HORDE_BASE', ANSEL_BASE . '/..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/MIME/Magic.php';
require_once 'MDB2.php';
$config = Horde::getDriverConfig('datatree', 'sql');
unset($config['charset']);
$mdb2 = &MDB2::factory($config);
$vfs = &VFS::singleton($conf['vfs']['type'], Horde::getDriverConfig('vfs', $conf['vfs']['type']));
$views = array('full', 'screen', 'thumb', 'mini');

// Get the transition data.
$transitions = $mdb2->queryAll('select datatree_id, image_id, vfsname from ansel_datatree_transition', null, MDB2_FETCHMODE_ASSOC);

$map = array();
while (count($transitions)) {
    $transitions = _transition($transitions);
}

// Fix default images.
echo "Getting galleries to fix default image ids.\n";
$sql = "select distinct d.datatree_id from horde_datatree_attributes da left join horde_datatree d on da.datatree_id = d.datatree_id where da.attribute_name = 'default-type' and d.group_uid = 'horde.shares.ansel'";
$ids = $mdb2->queryCol($sql);
if (!is_a($ids, 'PEAR_Error')) {
    foreach ($ids as $galleryId) {
        $gallery = &$ansel_shares->getGallery($galleryId);
        if (!is_a($gallery, 'PEAR_Error') && isset($map[$gallery->get('default')])) {
            $gallery->set('default', $map[$gallery->get('default')], true);
        }
    }
}

echo wordwrap("\nFinished renaming files. After you test your galleries, you can run 2006-11-27_cleanup.php to clean out the bits of your database that aren't needed anymore. You may also want to run scripts/garbage_collection.php to clear out any stray datatree bits.\n");

/**
 */
function _transition($transitions)
{
    echo "Starting run, transitioning " . count($transitions) . " images ...";
    $deferred = array();
    foreach ($transitions as $transition) {
        $datatree_id = (int)$transition['datatree_id'];
        $image_id = (int)$transition['image_id'];
        $GLOBALS['map'][$datatree_id] = $image_id;
        $vfsname = $transition['vfsname'];

        $vfsbits = explode('.', $vfsname, 2);
        $vfsbits[0] = $image_id;
        $newvfsname = implode('.', $vfsbits);

        // Don't overwrite an existing file.
        if ($GLOBALS['vfs']->exists(_vpath($image_id), $newvfsname)) {
            $deferred[] = $transition;
            continue;
        }

        foreach ($GLOBALS['views'] as $view) {
            if ($GLOBALS['vfs']->exists(_vpath($datatree_id, $view), $vfsname)) {
                $GLOBALS['vfs']->rename(_vpath($datatree_id, $view), $vfsname, _vpath($image_id, $view), $newvfsname);
            }
            $GLOBALS['vfs']->deleteFolder('.horde/ansel/' . substr($datatree_id, -2), $view);
        }
        $GLOBALS['vfs']->deleteFolder('.horde/ansel', substr($datatree_id, -2));
    }

    echo " done.\n";
    return $deferred;
}

/**
 */
function _vpath($id, $view = 'full')
{
    return '.horde/ansel/' . substr($id, -2) . '/' . $view;
}
