#!/usr/local/bin/php
<?php
/**
 * $Horde: ansel/scripts/upgrades/2006-11-27_sql_images.php,v 1.10 2006/12/12 17:22:29 jan Exp $
 *
 * This script creates the new Ansel SQL structure. It is entirely
 * non-destructive so you can test this one out first.
 */

@define('AUTH_HANDLER', true);
@define('ANSEL_BASE', dirname(__FILE__) . '/../..');
@define('HORDE_BASE', ANSEL_BASE . '/..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/MIME/Magic.php';
require_once 'MDB2.php';
$config = Horde::getDriverConfig('datatree', 'sql');
unset($config['charset']);
$mdb2 = &MDB2::factory($config);
$vfs = &VFS::singleton($conf['vfs']['type'], Horde::getDriverConfig('vfs', $conf['vfs']['type']));

// Prepare statements.
$ansel_images = $mdb2->prepare('INSERT INTO ansel_images (image_id, gallery_id, image_filename, image_type, image_caption, image_uploaded, image_sort) VALUES (?, ?, ?, ?, ?, ?, ?)');
if (is_a($ansel_images, 'PEAR_Error')) {
    var_dump($ansel_images);
    exit;
}
$ansel_image_attr = $mdb2->prepare('INSERT INTO ansel_image_attributes (image_id, attr_name, attr_value) VALUES (?, ?, ?)');
if (is_a($ansel_image_attr, 'PEAR_Error')) {
    var_dump($ansel_image_attr);
    exit;
}
$ansel_datatree_transition = $mdb2->prepare('INSERT INTO ansel_datatree_transition (datatree_id, image_id, vfsname) VALUES (?, ?, ?)');
if (is_a($ansel_datatree_transition, 'PEAR_Error')) {
    var_dump($ansel_datatree_transition);
    exit;
}

// Get list of images.
$sql = "SELECT distinct d.datatree_id FROM horde_datatree_attributes da LEFT JOIN horde_datatree d ON da.datatree_id = d.datatree_id WHERE da.attribute_name = 'gallery' AND d.group_uid = 'horde.shares.ansel'";
$ids = $mdb2->queryCol($sql);

$copied = 0;
$skipped_err = 0;
$skipped_gid = 0;
$skipped_vfs = 0;
foreach ($ids as $datatreeId) {
    $all = $mdb2->queryAll('SELECT attribute_name, attribute_value FROM horde_datatree_attributes WHERE datatree_id = ' . (int)$datatreeId, null, MDB2_FETCHMODE_ASSOC, true);

    $image_filename = '';
    $image_caption = '';
    $image_type = '';
    $image_uploaded = null;
    $image_attr = array();
    $gallery_id = null;

    foreach ($all as $name => $value) {
        switch ($name) {
        // base image data
        case 'gallery':
            $gallery_id = $value;
            break;
        case 'filename':
            $image_filename = $value;
            break;
        case 'type':
            $image_type = $value;
            break;
        case 'description':
            $image_caption = $value;
            break;
        case 'date':
            $image_uploaded = $value;
            break;

        default:
            // put into the attributes table, but ignore blank values.
            if (strlen($value)) {
                $image_attr[$name] = $value;
            }
        }
    }

    $image_sort = $mdb2->queryOne('SELECT datatree_order FROM horde_datatree WHERE datatree_id = ' . (int)$datatreeId);
    if (is_a($image_sort, 'PEAR_Error') || is_null($image_sort)) {
        $image_sort = 0;
    }

    // Skip this image if we didn't get a gallery id.
    if (is_null($gallery_id)) {
        ++$skipped_gid;
        continue;
    }

    // See if this image exists in the VFS - no reason to convert if
    // it's an orphan.
    $vfsdir = '.horde/ansel/' . substr($datatreeId, -2) . '/full';
    $vfsname = $datatreeId;
    if ($ext = MIME_Magic::MIMEToExt($image_type)) {
        $vfsname .= '.' . $ext;
    }
    if (!$vfs->exists($vfsdir, $vfsname)) {
        if ($vfs->exists($vfsdir, $vfsname . '.jpg')) {
            $vfsname .= '.jpg';
            $image_type = 'image/jpeg';
        } elseif ($vfs->exists($vfsdir, $vfsname . '.png')) {
            $vfsname .= '.png';
            $image_type = 'image/png';
        } else {
            ++$skipped_vfs;
            continue;
        }
    }
    if (empty($image_type)) {
        $ext = substr($image_filename, strrpos($image_filename, '.') + 1);
        $vfsname .= '.' . $ext;
        $image_type = MIME_Magic::ExtToMIME($ext);
    }

    // Run inserts.
    $image_id = $mdb2->nextId('ansel_images');
    if (is_a($image_id, 'PEAR_Error')) {
        echo 'DEBUG (nextId): ' . $image_id->getMessage() . "\n";
        ++$skipped_err;
        continue;
    }
    $result = $ansel_images->execute(array($image_id, $gallery_id, $image_filename, $image_type, $image_caption, $image_uploaded, $image_sort));
    if (is_a($result, 'PEAR_Error')) {
        echo 'DEBUG (ansel_images): ' . $result->getMessage() . "\n";
        var_dump($result); exit;
        ++$skipped_err;
        continue;
    }

    foreach ($image_attr as $name => $value) {
        $result = $ansel_image_attr->execute(array($image_id, $name, $value));
        if (is_a($result, 'PEAR_Error')) {
            echo 'DEBUG (attributes): ' . $result->getMessage() . "\n";
            continue;
        }
    }

    // Put into the transition table.
    $ansel_datatree_transition->execute(array($datatreeId, $image_id, $vfsname));

    ++$copied;
}

echo wordwrap("\nCopied $copied images, skipped $skipped_gid for missing gallery ids, $skipped_vfs for missing VFS files, and $skipped_err for errors.\n\nThis script has not deleted or modified any data, only created the new structure. You should now run 2006-11-27_datatree_to_sql_renames.php to rename VFS data, but we strongly recommend you back up your data and test the new system before doing so.\n");
